@@ -50,7 +50,6 @@
 
 nsFontMetricsMac :: nsFontMetricsMac()
 {
-  NS_INIT_REFCNT();
   mFont = nsnull;
   mFontNum = BAD_FONT_NUM;
   mFontMapping = nsnull;
@@ -96,7 +95,7 @@ NS_IMETHODIMP nsFontMetricsMac::Init(const nsFont& aFont, nsIAtom* aLangGroup, n
   GetFontInfo(&fInfo);
   
   float  dev2app;
-  mContext->GetDevUnitsToAppUnits(dev2app);
+  dev2app = mContext->DevUnitsToAppUnits();
 
   mLeading    = NSToCoordRound(float(fInfo.leading) * dev2app);
   mEmAscent   = NSToCoordRound(float(fInfo.ascent) * dev2app);
@@ -108,8 +107,18 @@ NS_IMETHODIMP nsFontMetricsMac::Init(const nsFont& aFont, nsIAtom* aLangGroup, n
   mMaxDescent = mEmDescent;
 
   mMaxAdvance = NSToCoordRound(float(::CharWidth('M')) * dev2app);	// don't use fInfo.widMax here
+  mAveCharWidth = NSToCoordRound(float(::CharWidth('x')) * dev2app);	
   mSpaceWidth = NSToCoordRound(float(::CharWidth(' ')) * dev2app);
 
+  Point frac;
+  frac.h = frac.v = 1;
+  unsigned char x = 'x';
+  short ascent;
+  if (noErr == ::OutlineMetrics(1, &x, frac, frac, &ascent, 0, 0, 0, 0))
+    mXHeight = NSToCoordRound(float(ascent) * dev2app);
+  else
+    mXHeight = NSToCoordRound(float(mMaxAscent) * 0.71f); // 0.71 = 5 / 7
+
   return NS_OK;
 }
 
@@ -257,9 +266,7 @@ nsFontMetricsMac::Destroy()
 NS_IMETHODIMP
 nsFontMetricsMac :: GetXHeight(nscoord& aResult)
 {
-  float  dev2app;
-  mContext->GetDevUnitsToAppUnits(dev2app);
-  aResult = NSToCoordRound(float(mMaxAscent * 0.71f));		// 0.71 = 5 / 7
+  aResult = mXHeight;
   return NS_OK;
 }
 
@@ -267,7 +274,7 @@ NS_IMETHODIMP
 nsFontMetricsMac :: GetSuperscriptOffset(nscoord& aResult)
 {
   float  dev2app;
-  mContext->GetDevUnitsToAppUnits(dev2app);
+  dev2app = mContext->DevUnitsToAppUnits();
   aResult = NSToCoordRound(float(mMaxAscent / 2) - dev2app);
   return NS_OK;
 }
@@ -276,7 +283,7 @@ NS_IMETHODIMP
 nsFontMetricsMac :: GetSubscriptOffset(nscoord& aResult)
 {
   float  dev2app;
-  mContext->GetDevUnitsToAppUnits(dev2app);
+  dev2app = mContext->DevUnitsToAppUnits();
   aResult = NSToCoordRound(float(mMaxAscent / 2) - dev2app);
   return NS_OK;
 }
@@ -285,7 +292,7 @@ NS_IMETHODIMP
 nsFontMetricsMac :: GetStrikeout(nscoord& aOffset, nscoord& aSize)
 {
   float  dev2app;
-  mContext->GetDevUnitsToAppUnits(dev2app);
+  dev2app = mContext->DevUnitsToAppUnits();
   aOffset = NSToCoordRound(float(mMaxAscent / 2) - dev2app);
   aSize = NSToCoordRound(dev2app);
   return NS_OK;
@@ -295,7 +302,7 @@ NS_IMETHODIMP
 nsFontMetricsMac :: GetUnderline(nscoord& aOffset, nscoord& aSize)
 {
   float  dev2app;
-  mContext->GetDevUnitsToAppUnits(dev2app);
+  dev2app = mContext->DevUnitsToAppUnits();
   aOffset = -NSToCoordRound( dev2app );
   aSize   = NSToCoordRound( dev2app );
   return NS_OK;
@@ -365,6 +372,12 @@ NS_IMETHODIMP nsFontMetricsMac :: GetMaxAdvance(nscoord &aAdvance)
   return NS_OK;
 }
 
+NS_IMETHODIMP nsFontMetricsMac :: GetAveCharWidth(nscoord &aAveCharWidth)
+{
+  aAveCharWidth = mAveCharWidth;
+  return NS_OK;
+}
+
 nsresult nsFontMetricsMac :: GetSpaceWidth(nscoord &aSpaceWidth)
 {
   aSpaceWidth = mSpaceWidth;
