@@ -58,8 +58,10 @@
 #include "nsMsgIncomingServer.h"
 
 #include "nsICategoryManager.h"
+#include "nsXPCOM.h"
 #include "nsISupportsPrimitives.h"
 #include "nsIPrefService.h"
+#include "nsIPrefBranchInternal.h"
 
 // turn this on to see useful output
 #undef DEBUG_amds
@@ -79,6 +81,8 @@
 
 #define NC_RDF_ACCOUNTROOT "msgaccounts:/"
 
+#define PREF_SHOW_FAKE_ACCOUNT "mailnews.fakeaccount.show"
+
 typedef struct _serverCreationParams {
   nsISupportsArray *serverArray;
   nsIRDFService *rdfService;
@@ -102,6 +106,7 @@ nsIRDFResource* nsMsgAccountManagerDataSource::kNC_PageTag=nsnull;
 nsIRDFResource* nsMsgAccountManagerDataSource::kNC_IsDefaultServer=nsnull;
 nsIRDFResource* nsMsgAccountManagerDataSource::kNC_SupportsFilters=nsnull;
 nsIRDFResource* nsMsgAccountManagerDataSource::kNC_CanGetMessages=nsnull;
+nsIRDFResource* nsMsgAccountManagerDataSource::kNC_CanGetIncomingMessages=nsnull;
 
 // containment
 nsIRDFResource* nsMsgAccountManagerDataSource::kNC_Child=nsnull;
@@ -144,6 +149,7 @@ nsCOMPtr<nsISupportsArray> nsMsgAccountManagerDataSource::mAccountRootArcsOut;
 #define NC_RDF_ISDEFAULTSERVER NC_NAMESPACE_URI "IsDefaultServer"
 #define NC_RDF_SUPPORTSFILTERS NC_NAMESPACE_URI "SupportsFilters"
 #define NC_RDF_CANGETMESSAGES NC_NAMESPACE_URI "CanGetMessages"
+#define NC_RDF_CANGETINCOMINGMESSAGES NC_NAMESPACE_URI "CanGetIncomingMessages"
 
 nsMsgAccountManagerDataSource::nsMsgAccountManagerDataSource()
 {
@@ -153,40 +159,69 @@ nsMsgAccountManagerDataSource::nsMsgAccountManagerDataSource()
   
   // do per-class initialization here
   if (gAccountManagerResourceRefCnt++ == 0) {
-      getRDFService()->GetResource(NC_RDF_CHILD, &kNC_Child);
-      getRDFService()->GetResource(NC_RDF_NAME, &kNC_Name);
-      getRDFService()->GetResource(NC_RDF_FOLDERTREENAME, &kNC_FolderTreeName);
-      getRDFService()->GetResource(NC_RDF_FOLDERTREESIMPLENAME, &kNC_FolderTreeSimpleName);
-      getRDFService()->GetResource(NC_RDF_NAME_SORT, &kNC_NameSort);
-      getRDFService()->GetResource(NC_RDF_FOLDERTREENAME_SORT, &kNC_FolderTreeNameSort);
-      getRDFService()->GetResource(NC_RDF_PAGETAG, &kNC_PageTag);
-      getRDFService()->GetResource(NC_RDF_ISDEFAULTSERVER, &kNC_IsDefaultServer);
-      getRDFService()->GetResource(NC_RDF_SUPPORTSFILTERS, &kNC_SupportsFilters);
-      getRDFService()->GetResource(NC_RDF_CANGETMESSAGES, &kNC_CanGetMessages);
-      getRDFService()->GetResource(NC_RDF_ACCOUNT, &kNC_Account);
-      getRDFService()->GetResource(NC_RDF_SERVER, &kNC_Server);
-      getRDFService()->GetResource(NC_RDF_IDENTITY, &kNC_Identity);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_MAIN, &kNC_PageTitleMain);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_SERVER, &kNC_PageTitleServer);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_COPIES, &kNC_PageTitleCopies);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_OFFLINEANDDISKSPACE, &kNC_PageTitleOfflineAndDiskSpace);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_DISKSPACE, &kNC_PageTitleDiskSpace);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_ADDRESSING, &kNC_PageTitleAddressing);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_ADVANCED, &kNC_PageTitleAdvanced);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_SMTP, &kNC_PageTitleSMTP);
-      getRDFService()->GetResource(NC_RDF_PAGETITLE_FAKEACCOUNT, &kNC_PageTitleFakeAccount);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_CHILD), &kNC_Child);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME), &kNC_Name);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME),
+                                   &kNC_FolderTreeName);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREESIMPLENAME),
+                                   &kNC_FolderTreeSimpleName);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_NAME_SORT), &kNC_NameSort);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_FOLDERTREENAME_SORT),
+                                   &kNC_FolderTreeNameSort);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETAG), &kNC_PageTag);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_ISDEFAULTSERVER),
+                                   &kNC_IsDefaultServer);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_SUPPORTSFILTERS),
+                                   &kNC_SupportsFilters);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANGETMESSAGES),
+                                   &kNC_CanGetMessages);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_CANGETINCOMINGMESSAGES),
+                                   &kNC_CanGetIncomingMessages);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_ACCOUNT), &kNC_Account);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_SERVER), &kNC_Server);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_IDENTITY), &kNC_Identity);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_MAIN),
+                                   &kNC_PageTitleMain);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_SERVER),
+                                   &kNC_PageTitleServer);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_COPIES),
+                                   &kNC_PageTitleCopies);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_OFFLINEANDDISKSPACE),
+                                   &kNC_PageTitleOfflineAndDiskSpace);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_DISKSPACE),
+                                   &kNC_PageTitleDiskSpace);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_ADDRESSING),
+                                   &kNC_PageTitleAddressing);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_ADVANCED),
+                                   &kNC_PageTitleAdvanced);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_SMTP),
+                                   &kNC_PageTitleSMTP);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_PAGETITLE_FAKEACCOUNT),
+                                   &kNC_PageTitleFakeAccount);
       
-      getRDFService()->GetResource(NC_RDF_ACCOUNTROOT, &kNC_AccountRoot);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_ACCOUNTROOT),
+                                   &kNC_AccountRoot);
 
       getRDFService()->GetLiteral(NS_LITERAL_STRING("true").get(),
                                   &kTrueLiteral);
       
       // eventually these need to exist in some kind of array
       // that's easily extensible
-      getRDFService()->GetResource(NC_RDF_SETTINGS, &kNC_Settings);
+      getRDFService()->GetResource(NS_LITERAL_CSTRING(NC_RDF_SETTINGS), &kNC_Settings);
       
       kDefaultServerAtom = NS_NewAtom("DefaultServer");
-    }
+   }
+
+   nsCOMPtr<nsIPrefBranchInternal> prefBranchInternal;
+   nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+   if (prefs) {
+     nsCOMPtr<nsIPrefBranch> prefBranch;
+     prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+     if (prefBranch) {
+       prefBranchInternal = do_QueryInterface(prefBranch);
+       prefBranchInternal->AddObserver(PREF_SHOW_FAKE_ACCOUNT, this, PR_FALSE);
+     }
+   }
 }
 
 nsMsgAccountManagerDataSource::~nsMsgAccountManagerDataSource()
@@ -207,6 +242,7 @@ nsMsgAccountManagerDataSource::~nsMsgAccountManagerDataSource()
       NS_IF_RELEASE(kNC_IsDefaultServer);
       NS_IF_RELEASE(kNC_SupportsFilters);
       NS_IF_RELEASE(kNC_CanGetMessages);
+      NS_IF_RELEASE(kNC_CanGetIncomingMessages);
       NS_IF_RELEASE(kNC_Account);
       NS_IF_RELEASE(kNC_Server);
       NS_IF_RELEASE(kNC_Identity);
@@ -255,7 +291,7 @@ nsMsgAccountManagerDataSource::Init()
     // get a weak ref to the account manager
     if (!mAccountManager) {
         am = do_GetService(NS_MSGACCOUNTMANAGER_CONTRACTID, &rv);
-        mAccountManager = getter_AddRefs(NS_GetWeakReference(am));
+        mAccountManager = do_GetWeakReference(am);
     } else
         am = do_QueryReferent(mAccountManager);
 
@@ -289,8 +325,8 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
                                          nsIRDFNode **target)
 {
   nsresult rv;
-  
-  
+    
+
   rv = NS_RDF_NO_VALUE;
 
   nsAutoString str;
@@ -331,14 +367,15 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
         if (NS_SUCCEEDED(rv))
           rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
         PRBool showFakeAccount;
-        rv = prefBranch->GetBoolPref("mailnews.fakeaccount.show", &showFakeAccount);
+        rv = prefBranch->GetBoolPref(PREF_SHOW_FAKE_ACCOUNT, &showFakeAccount);
         if (showFakeAccount) {
           nsCOMPtr<nsIStringBundleService> strBundleService = do_GetService(NS_STRINGBUNDLE_CONTRACTID, &rv);
           if (NS_FAILED(rv)) return rv;
+          nsCOMPtr<nsIStringBundle> bundle;
           rv = strBundleService->CreateBundle("chrome://messenger/locale/fakeAccount.properties",
-                                               getter_AddRefs(mStringBundle));
+                                               getter_AddRefs(bundle));
           if (NS_SUCCEEDED(rv))
-              mStringBundle->GetStringFromName(NS_LITERAL_STRING("prefPanel-fake-account").get(), 
+              bundle->GetStringFromName(NS_LITERAL_STRING("prefPanel-fake-account").get(), 
                                                getter_Copies(pageTitle));
         }
       }
@@ -366,10 +403,19 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
             // make sure the pointer math we're about to do is safe.
             NS_ENSURE_TRUE(sourceValue && (strlen(sourceValue) > strlen(NC_RDF_PAGETITLE_PREFIX)), NS_ERROR_UNEXPECTED);
 
-            nsCAutoString bundleURL;
-            bundleURL = "chrome://messenger/locale/";
-            bundleURL += "am-";
+            nsCOMPtr<nsIMsgAccountManager> am =
+              do_QueryReferent(mAccountManager, &rv);
+            NS_ENSURE_SUCCESS(rv, PR_FALSE);
+            
             // turn NC#PageTitlefoobar into foobar, so we can get the am-foobar.properties bundle
+            nsXPIDLCString chromePackageName;
+            rv = am->GetChromePackageName((sourceValue + strlen(NC_RDF_PAGETITLE_PREFIX)), getter_Copies(chromePackageName));
+            NS_ENSURE_SUCCESS(rv,rv);
+
+            nsCAutoString bundleURL;
+            bundleURL = "chrome://";
+            bundleURL += chromePackageName;
+            bundleURL += "/locale/am-";
             bundleURL += (sourceValue + strlen(NC_RDF_PAGETITLE_PREFIX));
             bundleURL += ".properties";
 
@@ -402,6 +448,8 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
       str = NS_LITERAL_STRING("am-advanced.xul");
     else if (source == kNC_PageTitleSMTP) 
       str = NS_LITERAL_STRING("am-smtp.xul");
+    else if (source == kNC_PageTitleFakeAccount) 
+      str = NS_LITERAL_STRING("am-fakeaccount.xul");
     else {
       nsCOMPtr<nsIMsgFolder> folder = do_QueryInterface(source, &rv);
       if (NS_SUCCEEDED(rv) && folder) {
@@ -443,28 +491,28 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
   else if ((property == kNC_NameSort) ||
            (property == kNC_FolderTreeNameSort)) {
 
+    // order for the folder pane
+    // and for the account manager tree is:
+    //
+    // - default mail account
+    // - <other mail accounts>
+    // - "Local Folders" account
+    // - news accounts
+    // - smtp settings (note, this is only in account manager tree)
+    // - fake account
+
     // make sure we're handling a root folder that is a server
     nsCOMPtr<nsIMsgIncomingServer> server;
     rv = getServerForFolderNode(source, getter_AddRefs(server));
 
-    // only answer for servers!
-    if (NS_FAILED(rv) || !server)
-        return NS_RDF_NO_VALUE;
-
-    // order is:
-    // - default account
-    // - <other servers>
-    // - Local Folders
-    // - news
-    
-    PRInt32 accountNum;
-    nsCOMPtr<nsIMsgAccountManager> am =
+    if (NS_SUCCEEDED(rv) && server) {     
+      PRInt32 accountNum;
+      nsCOMPtr<nsIMsgAccountManager> am =
         do_QueryReferent(mAccountManager);
-
-    if (isDefaultServer(server))
+      
+      if (isDefaultServer(server))
         str = NS_LITERAL_STRING("0000");
-    else {
-    
+      else {
         rv = am->FindServerIndex(server, &accountNum);
         if (NS_FAILED(rv)) return rv;
         
@@ -473,13 +521,52 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
         server->GetType(getter_Copies(serverType));
         
         if (nsCRT::strcasecmp(serverType, "none")==0)
-            accountNum += 2000;
+          accountNum += 2000;
         else if (nsCRT::strcasecmp(serverType, "nntp")==0)
-            accountNum += 3000;
+          accountNum += 3000;
         else
-            accountNum += 1000;     // default is to appear at the top
+          accountNum += 1000;     // default is to appear at the top
         
         str.AppendInt(accountNum);
+      }
+    }
+    else {
+      const char *sourceValue;
+      rv = source->GetValueConst(&sourceValue);
+      NS_ENSURE_SUCCESS(rv, NS_RDF_NO_VALUE);
+      
+      // if this is a page (which we determine by the prefix of the URI)
+      // we want to generate a sort value
+      // so that we can sort the categories in the account manager tree
+      // (or the folder pane)
+      //
+      // otherwise, return NS_RDF_NO_VALUE
+      // so that the folder data source will take care of it.
+      if (sourceValue && (strncmp(sourceValue, NC_RDF_PAGETITLE_PREFIX, strlen(NC_RDF_PAGETITLE_PREFIX)) == 0)) {
+        if (source == kNC_PageTitleSMTP)
+          str = NS_LITERAL_STRING("4000");
+        else if (source == kNC_PageTitleFakeAccount)
+          str = NS_LITERAL_STRING("5000");
+        else if (source == kNC_PageTitleServer)
+          str = NS_LITERAL_STRING("1");
+        else if (source == kNC_PageTitleCopies)
+          str = NS_LITERAL_STRING("2");
+        else if (source == kNC_PageTitleAddressing)
+          str = NS_LITERAL_STRING("3");
+        else if (source == kNC_PageTitleOfflineAndDiskSpace)
+          str = NS_LITERAL_STRING("4");
+        else if (source == kNC_PageTitleDiskSpace)
+          str = NS_LITERAL_STRING("4");
+        else {
+          // allow for the accountmanager to be dynamically extended
+          // all the other pages come after the standard ones
+          // server, copies, addressing, disk space (or offline & disk space)
+          str.AssignWithConversion(sourceValue);
+        }
+      }
+      else {
+        return NS_RDF_NO_VALUE;
+      }
     }
   }
 
@@ -522,6 +609,14 @@ nsMsgAccountManagerDataSource::GetTarget(nsIRDFResource *source,
       if (canGetMessages(server))
           str = NS_LITERAL_STRING("true");
   }
+  else if (property == kNC_CanGetIncomingMessages) {
+      nsCOMPtr<nsIMsgIncomingServer> server;
+      rv = getServerForFolderNode(source, getter_AddRefs(server));
+      if (NS_FAILED(rv) || !server) return NS_RDF_NO_VALUE;
+
+      if (canGetIncomingMessages(server))
+          str = NS_LITERAL_STRING("true");
+  }
   else if (property == kNC_PageTitleFakeAccount) {
     if (source == kNC_PageTitleFakeAccount)
       str = NS_LITERAL_STRING("true");
@@ -606,10 +701,12 @@ nsMsgAccountManagerDataSource::createRootResources(nsIRDFResource *property,
         printf("GetTargets(): added %d servers on %s\n", nodecount,
                (const char*)property_arc);
 #endif
-        // for the "settings" arc, we also want to do an SMTP tag
+        // for the "settings" arc, we also want to add SMTP setting and the fake account (if required)
         if (property == kNC_Settings) {
             aNodeArray->AppendElement(kNC_PageTitleSMTP);
-        }
+            if (IsFakeAccountRequired())
+              aNodeArray->AppendElement(kNC_PageTitleFakeAccount);            
+        } 
         else if (property == kNC_Child && IsFakeAccountRequired()) {
             aNodeArray->AppendElement(kNC_PageTitleFakeAccount);
         }
@@ -636,13 +733,13 @@ nsMsgAccountManagerDataSource::appendGenericSettingsResources(nsIMsgIncomingServ
   rv = catman->EnumerateCategory(MAILNEWS_ACCOUNTMANAGER_EXTENSIONS, getter_AddRefs(e));
   if(NS_SUCCEEDED(rv) && e) {
     while (PR_TRUE) {
-      nsCOMPtr<nsISupportsString> catEntry;
+      nsCOMPtr<nsISupportsCString> catEntry;
       rv = e->GetNext(getter_AddRefs(catEntry));
       if (NS_FAILED(rv) || !catEntry) 
         break;
 
-      nsXPIDLCString entryString;
-      rv = catEntry->GetData(getter_Copies(entryString));
+      nsCAutoString entryString;
+      rv = catEntry->GetData(entryString);
       if (NS_FAILED(rv))
          break;
 
@@ -687,7 +784,7 @@ nsMsgAccountManagerDataSource::appendGenericSetting(const char *name, nsISupport
   resourceStr = NC_RDF_PAGETITLE_PREFIX;
   resourceStr += name;
 
-  nsresult rv = getRDFService()->GetResource(resourceStr.get(), getter_AddRefs(resource));
+  nsresult rv = getRDFService()->GetResource(resourceStr, getter_AddRefs(resource));
   NS_ENSURE_SUCCESS(rv,rv);
 
   // AppendElement will addref.
@@ -791,14 +888,14 @@ nsMsgAccountManagerDataSource::createServerResources(nsISupports *element,
   nsresult rv;
   // get parameters out of the data argument
   serverCreationParams *params = (serverCreationParams*)data;
-  nsCOMPtr<nsISupportsArray> servers = dont_QueryInterface(params->serverArray);
-  nsCOMPtr<nsIRDFService> rdf = dont_QueryInterface(params->rdfService);
+  nsCOMPtr<nsISupportsArray> servers = params->serverArray;
+  nsCOMPtr<nsIRDFService> rdf = params->rdfService;
 
   // the server itself is in the element argument
   nsCOMPtr<nsIMsgIncomingServer> server = do_QueryInterface(element, &rv);
   if (NS_FAILED(rv)) return PR_TRUE;
 
-	nsCOMPtr <nsIFolder> serverFolder;
+	nsCOMPtr <nsIMsgFolder> serverFolder;
 	rv = server->GetRootFolder(getter_AddRefs(serverFolder));
 	if(NS_FAILED(rv)) return PR_TRUE;
 
@@ -928,7 +1025,7 @@ nsMsgAccountManagerDataSource::HasAssertion(nsIRDFResource *aSource,
   //
   // short-circuit on property, so objects like filters, etc, don't get queried
   else if (aProperty == kNC_IsDefaultServer || aProperty == kNC_CanGetMessages || 
-           aProperty == kNC_SupportsFilters) {
+           aProperty == kNC_CanGetIncomingMessages || aProperty == kNC_SupportsFilters) {
     nsCOMPtr<nsIMsgIncomingServer> server;
     rv = getServerForFolderNode(aSource, getter_AddRefs(server));
     if (NS_SUCCEEDED(rv) && server)
@@ -952,27 +1049,17 @@ nsMsgAccountManagerDataSource::HasAssertionServer(nsIMsgIncomingServer *aServer,
                                                   PRBool aTruthValue,
                                                   PRBool *_retval)
 {
-
-    if (aProperty == kNC_IsDefaultServer) {
-        if (aTarget == kTrueLiteral)
-            *_retval = isDefaultServer(aServer);
-        else
-            *_retval = !isDefaultServer(aServer);
-
-    } else if (aProperty == kNC_SupportsFilters) {
-        if (aTarget == kTrueLiteral) {
-            *_retval = supportsFilters(aServer);
-        } else
-            *_retval = !supportsFilters(aServer);
-    } else if (aProperty == kNC_CanGetMessages) {
-        if (aTarget == kTrueLiteral) {
-            *_retval = canGetMessages(aServer);
-        } else
-            *_retval = !canGetMessages(aServer);
-    } else {
-        *_retval = PR_FALSE;
-    }
-    return NS_OK;
+  if (aProperty == kNC_IsDefaultServer)
+    *_retval = (aTarget == kTrueLiteral) ? isDefaultServer(aServer) : !isDefaultServer(aServer);
+  else if (aProperty == kNC_SupportsFilters)
+    *_retval = (aTarget == kTrueLiteral) ? supportsFilters(aServer) : !supportsFilters(aServer);
+  else if (aProperty == kNC_CanGetMessages) 
+   *_retval = (aTarget == kTrueLiteral) ? canGetMessages(aServer) : !canGetMessages(aServer); 
+  else if (aProperty == kNC_CanGetIncomingMessages) 
+    *_retval = (aTarget == kTrueLiteral) ? canGetIncomingMessages(aServer) : !canGetIncomingMessages(aServer);
+  else
+    *_retval = PR_FALSE;
+  return NS_OK;
 }
 
 PRBool
@@ -990,7 +1077,7 @@ nsMsgAccountManagerDataSource::isDefaultServer(nsIMsgIncomingServer *aServer)
     NS_ENSURE_SUCCESS(rv, PR_FALSE);
     if (!defaultAccount) return PR_FALSE;
 
-    // in some wierd case that there is no default and they asked
+    // in some weird case that there is no default and they asked
     // for the default
     nsCOMPtr<nsIMsgIncomingServer> defaultServer;
     rv = defaultAccount->GetIncomingServer(getter_AddRefs(defaultServer));
@@ -1018,23 +1105,41 @@ nsMsgAccountManagerDataSource::supportsFilters(nsIMsgIncomingServer *aServer)
 PRBool
 nsMsgAccountManagerDataSource::canGetMessages(nsIMsgIncomingServer *aServer)
 {
-      nsresult rv; 
+  nsXPIDLCString type;
+  nsresult rv = aServer->GetType(getter_Copies(type));
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+  
+  nsCAutoString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
+  contractid.Append(type);
+  
+  nsCOMPtr<nsIMsgProtocolInfo> protocolInfo =
+    do_GetService(contractid.get(), &rv);
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+  
+  PRBool canGetMessages = PR_FALSE;
+  protocolInfo->GetCanGetMessages(&canGetMessages);
+  
+  return canGetMessages;
+}
 
-      nsXPIDLCString type;
-      rv = aServer->GetType(getter_Copies(type));
-      NS_ENSURE_SUCCESS(rv, PR_FALSE);
-
-      nsCAutoString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
-      contractid.Append(type);
-
-      nsCOMPtr<nsIMsgProtocolInfo> protocolInfo =
-           do_GetService(contractid.get(), &rv);
-      NS_ENSURE_SUCCESS(rv, PR_FALSE);
-
-      PRBool canGetMessages = PR_FALSE;
-      protocolInfo->GetCanGetMessages(&canGetMessages);
-
-      return canGetMessages;
+PRBool
+nsMsgAccountManagerDataSource::canGetIncomingMessages(nsIMsgIncomingServer *aServer)
+{
+  nsXPIDLCString type;
+  nsresult rv = aServer->GetType(getter_Copies(type));
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+  
+  nsCAutoString contractid(NS_MSGPROTOCOLINFO_CONTRACTID_PREFIX);
+  contractid.Append(type);
+  
+  nsCOMPtr<nsIMsgProtocolInfo> protocolInfo =
+    do_GetService(contractid.get(), &rv);
+  NS_ENSURE_SUCCESS(rv, PR_FALSE);
+  
+  PRBool canGetIncomingMessages = PR_FALSE;
+  protocolInfo->GetCanGetIncomingMessages(&canGetIncomingMessages);
+  
+  return canGetIncomingMessages;
 }
 
 nsresult
@@ -1154,7 +1259,7 @@ nsMsgAccountManagerDataSource::OnServerLoaded(nsIMsgIncomingServer* aServer)
 {
   nsresult rv;
 
-  nsCOMPtr<nsIFolder> serverFolder;
+  nsCOMPtr<nsIMsgFolder> serverFolder;
   rv = aServer->GetRootFolder(getter_AddRefs(serverFolder));
   if (NS_FAILED(rv)) return rv;
 
@@ -1174,9 +1279,11 @@ nsMsgAccountManagerDataSource::OnServerLoaded(nsIMsgIncomingServer* aServer)
   PRBool fakeAccountServer;
   IsIncomingServerForFakeAccount(aServer, &fakeAccountServer);
 
-  if (fakeAccountServer)
+  if (fakeAccountServer) {
     NotifyObservers(kNC_AccountRoot, kNC_Child, kNC_PageTitleFakeAccount, PR_FALSE, PR_FALSE);
-  
+    NotifyObservers(kNC_AccountRoot, kNC_Settings, kNC_PageTitleFakeAccount, PR_FALSE, PR_FALSE);
+  }
+
   return NS_OK;
 }
 
@@ -1185,7 +1292,7 @@ nsMsgAccountManagerDataSource::OnServerUnloaded(nsIMsgIncomingServer* aServer)
 {
   nsresult rv;
   
-  nsCOMPtr<nsIFolder> serverFolder;
+  nsCOMPtr<nsIMsgFolder> serverFolder;
   rv = aServer->GetRootFolder(getter_AddRefs(serverFolder));
   if (NS_FAILED(rv)) return rv;
 
@@ -1268,7 +1375,7 @@ nsMsgAccountManagerDataSource::OnItemBoolPropertyChanged(nsISupports *aItem,
 }
 
 nsresult
-nsMsgAccountManagerDataSource::OnItemEvent(nsIFolder *, nsIAtom *)
+nsMsgAccountManagerDataSource::OnItemEvent(nsIMsgFolder *, nsIAtom *)
 {
     return NS_OK;
 }
@@ -1289,7 +1396,7 @@ nsMsgAccountManagerDataSource::IsFakeAccountRequired()
     rv = prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
 
   PRBool showFakeAccount;
-  rv = prefBranch->GetBoolPref("mailnews.fakeaccount.show", &showFakeAccount);
+  rv = prefBranch->GetBoolPref(PREF_SHOW_FAKE_ACCOUNT, &showFakeAccount);
 
   if (!showFakeAccount)
     return PR_FALSE;
@@ -1333,7 +1440,7 @@ nsMsgAccountManagerDataSource::IsIncomingServerForFakeAccount(nsIMsgIncomingServ
   rv = aServer->GetHostName(getter_Copies(hostname));
   NS_ENSURE_SUCCESS(rv,rv);
 
-  *aResult = (strcmp(hostname.get(), fakeAccountHostName) == 0);
+  *aResult = (strcmp(hostname.get(), fakeAccountHostName.get()) == 0);
   return NS_OK;
 }
 
@@ -1351,3 +1458,31 @@ nsMsgAccountManagerDataSource::GetFakeAccountHostName(char **aHostName)
 
   return NS_OK;
 }
+
+NS_IMETHODIMP
+nsMsgAccountManagerDataSource::Observe(nsISupports *aSubject, const char *aTopic, const PRUnichar *aData)
+{
+  nsMsgRDFDataSource::Observe(aSubject, aTopic, aData);
+  
+  if (!strcmp(aTopic, NS_PREFBRANCH_PREFCHANGE_TOPIC_ID)) {
+    nsDependentString prefName(aData);
+    if (prefName.Equals(NS_LITERAL_STRING(PREF_SHOW_FAKE_ACCOUNT))) {
+      NotifyObservers(kNC_AccountRoot, kNC_Child, kNC_PageTitleFakeAccount, PR_FALSE, PR_FALSE);
+      NotifyObservers(kNC_AccountRoot, kNC_Settings, kNC_PageTitleFakeAccount, PR_FALSE, PR_FALSE);
+    }
+  }
+  else if (!strcmp(aTopic, NS_XPCOM_SHUTDOWN_OBSERVER_ID)) {
+    nsCOMPtr<nsIPrefBranchInternal> prefBranchInternal;
+    nsCOMPtr<nsIPrefService> prefs = do_GetService(NS_PREFSERVICE_CONTRACTID);
+    if (prefs) {
+      nsCOMPtr<nsIPrefBranch> prefBranch;
+      prefs->GetBranch(nsnull, getter_AddRefs(prefBranch));
+      if (prefBranch) {
+        prefBranchInternal = do_QueryInterface(prefBranch);
+        prefBranchInternal->RemoveObserver(PREF_SHOW_FAKE_ACCOUNT, this);
+      }
+    }
+  }
+  return NS_OK;
+}
+
