@@ -401,6 +401,8 @@ MimeMultipartSigned_parse_line (char *line, PRInt32 length, MimeObject *obj)
              !nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE3) ||
              !nsCRT::strcasecmp(encoding.get(), ENCODING_UUENCODE4))
       fn = &MimeUUDecoderInit;
+    else if (!nsCRT::strcasecmp(encoding.get(), ENCODING_YENCODE))
+      fn = &MimeYDecoderInit;
 		if (fn)
 		  {
 			sig->sig_decoder_data =
@@ -600,7 +602,7 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
   int status = 0;
   MimeObject *body;
 
-  PR_ASSERT(sig->crypto_closure);
+  NS_ASSERTION(sig->crypto_closure, "no crypto closure");
 
   /* Emit some HTML saying whether the signature was cool.
 	 But don't emit anything if in FO_QUOTE_MESSAGE mode.
@@ -692,10 +694,7 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
           char *cset = MimeHeaders_get_parameter (ct, "charset", NULL, NULL);
           if (cset) {
             mimeEmitterUpdateCharacterSet(obj->options, cset);
-            if (!nsCRT::strcasecmp(cset, "us-ascii"))
-              SetMailCharacterSetToMsgWindow(obj, NS_LITERAL_STRING("ISO-8859-1").get());
-            else
-              SetMailCharacterSetToMsgWindow(obj, NS_ConvertASCIItoUCS2(cset).get());
+            SetMailCharacterSetToMsgWindow(obj, cset);
             PR_Free(cset);
           }
           PR_Free(ct);
@@ -731,7 +730,7 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
 		  status = MimePartBufferRead (sig->part_buffer,
 							   /* The (nsresult (*) ...) cast is to turn the
 								  `void' argument into `MimeObject'. */
-							   ((nsresult (*) (char *, PRInt32, void *))
+							   ((nsresult (*) (const char *, PRInt32, void *))
 							   body->options->decompose_file_output_fn),
 							   body->options->stream_closure);
 	  else
@@ -740,7 +739,7 @@ MimeMultipartSigned_emit_child (MimeObject *obj)
 	  status = MimePartBufferRead (sig->part_buffer,
 							   /* The (nsresult (*) ...) cast is to turn the
 								  `void' argument into `MimeObject'. */
-							   ((nsresult (*) (char *, PRInt32, void *))
+							   ((nsresult (*) (const char *, PRInt32, void *))
 								body->clazz->parse_buffer),
 								body);
 	  if (status < 0) return status;
