@@ -57,7 +57,6 @@ struct RedirEntry {
     PRBool dropChromePrivs; // if PR_TRUE, the page will not have chrome privileges
 };
 
-// if you add something here, make sure you update the total
 /*
   Entries with dropChromePrivs == PR_FALSE will run with chrome
   privileges. This is potentially dangerous. Please use PR_TRUE
@@ -67,11 +66,14 @@ struct RedirEntry {
  */
 static RedirEntry kRedirMap[] = {
     { "credits", "http://www.mozilla.org/credits/", PR_TRUE },
-    { "mozilla", "chrome://global/content/mozilla.html", PR_TRUE },
-    { "plugins", "chrome://global/content/plugins.html", PR_TRUE },
-    { "config", "chrome://global/content/config.xul", PR_FALSE }
+    { "mozilla", "chrome://global/content/mozilla.xhtml", PR_TRUE },
+    { "plugins", "chrome://communicator/content/plugins.html", PR_FALSE },
+    { "config", "chrome://global/content/config.xul", PR_FALSE },
+    { "logo", "chrome://global/content/logo.gif", PR_TRUE },
+    { "buildconfig", "chrome://global/content/buildconfig.html", PR_TRUE },
+    { "about", "chrome://global/content/aboutAbout.html", PR_FALSE }
 };
-static const int kRedirTotal = 4; // sizeof(kRedirMap)/sizeof(*kRedirMap)
+static const int kRedirTotal = sizeof(kRedirMap)/sizeof(*kRedirMap);
 
 NS_IMETHODIMP
 nsAboutRedirector::NewChannel(nsIURI *aURI, nsIChannel **result)
