@@ -35,7 +35,6 @@
 #include "nsIComponentManager.h"
 #include "nsIServiceManager.h"
 #include "nsIStringBundle.h"
-#include "nsIPref.h"
 #include "nsIInterfaceRequestor.h"
 #include "nsIInterfaceRequestorUtils.h"
 #include "nsIX509Cert.h"
@@ -43,116 +42,47 @@
 #include "nsILocaleService.h"
 #include "nsIDateTimeFormat.h"
 #include "nsDateTimeFormatCID.h"
-#include "nsIWindowWatcher.h"
 
 #include "nsNSSDialogs.h"
 #include "nsPKIParamBlock.h"
 #include "nsIKeygenThread.h"
+#include "nsNSSDialogHelper.h"
+#include "nsIX509CertValidity.h"
+#include "nsICRLInfo.h"
 
 #define PIPSTRING_BUNDLE_URL "chrome://pippki/locale/pippki.properties"
-#define STRING_BUNDLE_URL    "chrome://communicator/locale/security.properties"
-
-#define ENTER_SITE_PREF      "security.warn_entering_secure"
-#define WEAK_SITE_PREF       "security.warn_entering_weak"
-#define LEAVE_SITE_PREF      "security.warn_leaving_secure"
-#define MIXEDCONTENT_PREF    "security.warn_viewing_mixed"
-#define INSECURE_SUBMIT_PREF "security.warn_submit_insecure"
 
 static NS_DEFINE_CID(kCStringBundleServiceCID,  NS_STRINGBUNDLESERVICE_CID);
-static NS_DEFINE_CID(kPrefCID, NS_PREF_CID);
 static NS_DEFINE_CID(kDateTimeFormatCID, NS_DATETIMEFORMAT_CID);
 
-/**
- * Common class that uses the window watcher service to open a
- * standard dialog, with or without a parent context. The params
- * parameter can be an nsISupportsArray so any number of additional
- * arguments can be used.
- */
-class nsNSSDialogHelper
-{
-public:
-  const static char *kDefaultOpenWindowParam;
-  //The params is going to be either a nsIPKIParamBlock or
-  //nsIDialogParamBlock
-  static nsresult openDialog(
-                  nsIDOMWindowInternal *window,
-                  const char *url,
-                  nsISupports *params);
-};
-
-const char* nsNSSDialogHelper::kDefaultOpenWindowParam = "centerscreen,chrome,modal,titlebar";
-
-nsresult
-nsNSSDialogHelper::openDialog(
-    nsIDOMWindowInternal *window,
-    const char *url,
-    nsISupports *params)
-{
-  nsresult rv;
-  nsCOMPtr<nsIWindowWatcher> windowWatcher = 
-           do_GetService("@mozilla.org/embedcomp/window-watcher;1", &rv);
-  if (NS_FAILED(rv)) return rv;
-
-  nsIDOMWindowInternal *parent = window;
-
-  nsCOMPtr<nsIDOMWindowInternal> activeParent;
-  if (!parent) {
-    nsCOMPtr<nsIDOMWindow> active;
-    windowWatcher->GetActiveWindow(getter_AddRefs(active));
-    if (active) {
-      active->QueryInterface(NS_GET_IID(nsIDOMWindowInternal), getter_AddRefs(activeParent));
-      parent = activeParent;
-    }
-  }
-
-  nsCOMPtr<nsIDOMWindow> newWindow;
-  rv = windowWatcher->OpenWindow(parent,
-                                 url,
-                                 "_blank",
-                                 nsNSSDialogHelper::kDefaultOpenWindowParam,
-                                 params,
-                                 getter_AddRefs(newWindow));
-  return rv;
-}
-
 /* ==== */
 static NS_DEFINE_CID(kPKIParamBlockCID, NS_PKIPARAMBLOCK_CID);
 
 nsNSSDialogs::nsNSSDialogs()
 {
-  NS_INIT_ISUPPORTS();
 }
 
 nsNSSDialogs::~nsNSSDialogs()
 {
 }
 
-NS_IMPL_THREADSAFE_ISUPPORTS10(nsNSSDialogs, nsINSSDialogs, 
-                                             nsITokenPasswordDialogs,
-                                             nsISecurityWarningDialogs,
-                                             nsIBadCertListener,
-                                             nsICertificateDialogs,
-                                             nsIClientAuthDialogs,
-                                             nsICertPickDialogs,
-                                             nsITokenDialogs,
-                                             nsIDOMCryptoDialogs,
-                                             nsIGeneratingKeypairInfoDialogs);
+NS_IMPL_THREADSAFE_ISUPPORTS8(nsNSSDialogs, nsITokenPasswordDialogs,
+                                            nsIBadCertListener,
+                                            nsICertificateDialogs,
+                                            nsIClientAuthDialogs,
+                                            nsICertPickDialogs,
+                                            nsITokenDialogs,
+                                            nsIDOMCryptoDialogs,
+                                            nsIGeneratingKeypairInfoDialogs)
 
 nsresult
 nsNSSDialogs::Init()
 {
   nsresult rv;
 
-  mPref = do_GetService(kPrefCID, &rv);
-  if (NS_FAILED(rv)) return rv;
-
   nsCOMPtr<nsIStringBundleService> service = do_GetService(kCStringBundleServiceCID, &rv);
   if (NS_FAILED(rv)) return rv;
   
-  rv = service->CreateBundle(STRING_BUNDLE_URL,
-                             getter_AddRefs(mStringBundle));
-  if (NS_FAILED(rv)) return rv;
-
   rv = service->CreateBundle(PIPSTRING_BUNDLE_URL,
                              getter_AddRefs(mPIPStringBundle));
   return rv;
@@ -169,7 +99,7 @@ nsNSSDialogs::SetPassword(nsIInterfaceRequestor *ctx,
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
 
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
 
   // void ChangePassword(in wstring tokenName, out int status);
@@ -202,7 +132,7 @@ nsNSSDialogs::GetPassword(nsIInterfaceRequestor *ctx,
   *_canceled = PR_FALSE;
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
   // Set the token name in the window
   rv = block->SetString(1, tokenName);
@@ -224,12 +154,10 @@ nsNSSDialogs::GetPassword(nsIInterfaceRequestor *ctx,
   return rv;
 }
 
-/* boolean unknownIssuer (in nsITransportSecurityInfo socketInfo,
-                          in nsIX509Cert cert, out addType); */
 NS_IMETHODIMP
-nsNSSDialogs::UnknownIssuer(nsITransportSecurityInfo *socketInfo,
-                            nsIX509Cert *cert, PRInt16 *outAddType,
-                            PRBool *_retval)
+nsNSSDialogs::ConfirmUnknownIssuer(nsIInterfaceRequestor *socketInfo,
+                                   nsIX509Cert *cert, PRInt16 *outAddType,
+                                   PRBool *_retval)
 {
   nsresult rv;
   PRInt32 addType;
@@ -284,14 +212,10 @@ nsNSSDialogs::UnknownIssuer(nsITransportSecurityInfo *socketInfo,
   return NS_OK; 
 }
 
-/* boolean mismatchDomain (in nsITransportSecurityInfo socketInfo, 
-                           in wstring targetURL, 
-                           in nsIX509Cert cert); */
-
 NS_IMETHODIMP 
-nsNSSDialogs::MismatchDomain(nsITransportSecurityInfo *socketInfo, 
-                             const PRUnichar *targetURL, 
-                             nsIX509Cert *cert, PRBool *_retval) 
+nsNSSDialogs::ConfirmMismatchDomain(nsIInterfaceRequestor *socketInfo, 
+                                    const nsACString &targetURL, 
+                                    nsIX509Cert *cert, PRBool *_retval) 
 {
   nsresult rv;
 
@@ -303,7 +227,7 @@ nsNSSDialogs::MismatchDomain(nsITransportSecurityInfo *socketInfo,
     return NS_ERROR_FAILURE;
 
   nsCOMPtr<nsIDialogParamBlock> dialogBlock = do_QueryInterface(block);
-  rv = dialogBlock->SetString(1, targetURL);
+  rv = dialogBlock->SetString(1, NS_ConvertUTF8toUCS2(targetURL).get());
   if (NS_FAILED(rv))
     return rv;
 
@@ -328,11 +252,9 @@ nsNSSDialogs::MismatchDomain(nsITransportSecurityInfo *socketInfo,
   return NS_OK;  
 }
 
-/* boolean certExpired (in nsITransportSecurityInfo socketInfo, 
-                        in nsIX509Cert cert); */
 NS_IMETHODIMP 
-nsNSSDialogs::CertExpired(nsITransportSecurityInfo *socketInfo, 
-                          nsIX509Cert *cert, PRBool *_retval)
+nsNSSDialogs::ConfirmCertExpired(nsIInterfaceRequestor *socketInfo, 
+                                 nsIX509Cert *cert, PRBool *_retval)
 {
   nsresult rv;
   PRTime now = PR_Now();
@@ -371,10 +293,10 @@ nsNSSDialogs::CertExpired(nsITransportSecurityInfo *socketInfo,
 
   nsXPIDLString message1;
   nsXPIDLString title;
-  PRUnichar *commonName=nsnull;
-  nsString formattedDate;
+  nsAutoString commonName;
+  nsAutoString formattedDate;
 
-  rv = cert->GetCommonName(&commonName);
+  rv = cert->GetCommonName(commonName);
 
   nsIDateTimeFormat *aDateTimeFormat;
   rv = nsComponentManager::CreateInstance(kDateTimeFormatCID, NULL,
@@ -384,18 +306,14 @@ nsNSSDialogs::CertExpired(nsITransportSecurityInfo *socketInfo,
   aDateTimeFormat->FormatPRTime(nsnull, kDateFormatShort, 
                                 kTimeFormatNoSeconds, timeToUse, 
                                 formattedDate);
-  PRUnichar *formattedDatePR = ToNewUnicode(formattedDate);
-  const PRUnichar *formatStrings[2] = { commonName, formattedDatePR }; 
-  nsString keyString = NS_ConvertASCIItoUCS2(key);
-  nsString titleKeyString = NS_ConvertASCIItoUCS2(titleKey);
+  const PRUnichar *formatStrings[2] = { commonName.get(), formattedDate.get() }; 
+  NS_ConvertASCIItoUCS2 keyString(key);
+  NS_ConvertASCIItoUCS2 titleKeyString(titleKey);
   mPIPStringBundle->FormatStringFromName(keyString.get(), formatStrings, 
                                          2, getter_Copies(message1));
   mPIPStringBundle->FormatStringFromName(titleKeyString.get(), formatStrings,
                                          2, getter_Copies(title));
   
-  Recycle(commonName);
-  Recycle(formattedDatePR);
-
   nsCOMPtr<nsIDialogParamBlock> dialogBlock = do_QueryInterface(block);
   rv = dialogBlock->SetString(1,message1); 
   rv = dialogBlock->SetString(2,title);
@@ -422,15 +340,15 @@ nsNSSDialogs::CertExpired(nsITransportSecurityInfo *socketInfo,
 }
 
 NS_IMETHODIMP 
-nsNSSDialogs::CrlNextupdate(nsITransportSecurityInfo *socketInfo, 
-                          const PRUnichar * targetURL, nsIX509Cert *cert)
+nsNSSDialogs::NotifyCrlNextupdate(nsIInterfaceRequestor *socketInfo, 
+                                  const nsACString &targetURL, nsIX509Cert *cert)
 {
   nsresult rv;
 
   nsCOMPtr<nsIPKIParamBlock> block = do_CreateInstance(kPKIParamBlockCID);
   nsCOMPtr<nsIDialogParamBlock> dialogBlock = do_QueryInterface(block);
 
-  rv = dialogBlock->SetString(1, targetURL);
+  rv = dialogBlock->SetString(1, NS_ConvertUTF8toUCS2(targetURL).get());
   if (NS_FAILED(rv))
     return rv;
 
@@ -445,7 +363,7 @@ nsNSSDialogs::CrlNextupdate(nsITransportSecurityInfo *socketInfo,
 }
 
 NS_IMETHODIMP 
-nsNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICrlEntry *crl)
+nsNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICRLInfo *crl)
 {
   nsresult rv;
 
@@ -463,212 +381,15 @@ nsNSSDialogs::CrlImportStatusDialog(nsIInterfaceRequestor *ctx, nsICrlEntry *crl
   return NS_OK;
 }
 
-nsresult
-nsNSSDialogs::AlertEnteringSecure(nsIInterfaceRequestor *ctx)
-{
-  nsresult rv;
-
-  rv = AlertDialog(ctx, ENTER_SITE_PREF, 
-                   NS_LITERAL_STRING("EnterSecureMessage").get(),
-                   NS_LITERAL_STRING("EnterSecureShowAgain").get());
-
-  return rv;
-}
-
-nsresult
-nsNSSDialogs::AlertEnteringWeak(nsIInterfaceRequestor *ctx)
-{
-  nsresult rv;
-
-  rv = AlertDialog(ctx, WEAK_SITE_PREF,
-                   NS_LITERAL_STRING("WeakSecureMessage").get(),
-                   NS_LITERAL_STRING("WeakSecureShowAgain").get());
-
-  return rv;
-}
-
-nsresult
-nsNSSDialogs::AlertLeavingSecure(nsIInterfaceRequestor *ctx)
-{
-  nsresult rv;
-
-  rv = AlertDialog(ctx, LEAVE_SITE_PREF, 
-                   NS_LITERAL_STRING("LeaveSecureMessage").get(),
-                   NS_LITERAL_STRING("LeaveSecureShowAgain").get());
-
-  return rv;
-}
-
-
-nsresult
-nsNSSDialogs::AlertMixedMode(nsIInterfaceRequestor *ctx)
-{
-  nsresult rv;
-
-  rv = AlertDialog(ctx, MIXEDCONTENT_PREF, 
-                   NS_LITERAL_STRING("MixedContentMessage").get(),
-                   NS_LITERAL_STRING("MixedContentShowAgain").get());
-
-  return rv;
-}
-
-
-nsresult
-nsNSSDialogs::AlertDialog(nsIInterfaceRequestor *ctx, const char *prefName,
-                          const PRUnichar *dialogMessageName,
-                          const PRUnichar *showAgainName)
-{
-  nsresult rv;
-
-  // Get user's preference for this alert
-  PRBool prefValue;
-  rv = mPref->GetBoolPref(prefName, &prefValue);
-  if (NS_FAILED(rv)) prefValue = PR_TRUE;
-
-  // Stop if alert is not requested
-  if (!prefValue) return NS_OK;
-
-  // Get Prompt to use
-  nsCOMPtr<nsIPrompt> prompt = do_GetInterface(ctx);
-  if (!prompt) return NS_ERROR_FAILURE;
-
-  // Get messages strings from localization file
-  nsXPIDLString windowTitle, message, dontShowAgain;
-
-  mStringBundle->GetStringFromName(NS_LITERAL_STRING("Title").get(),
-                                   getter_Copies(windowTitle));
-  mStringBundle->GetStringFromName(dialogMessageName,
-                                   getter_Copies(message));
-  mStringBundle->GetStringFromName(showAgainName,
-                                   getter_Copies(dontShowAgain));
-  if (!windowTitle || !message || !dontShowAgain) return NS_ERROR_FAILURE;
-      
-  rv = prompt->AlertCheck(windowTitle, message, dontShowAgain, &prefValue);
-  if (NS_FAILED(rv)) return rv;
-      
-  if (!prefValue) {
-    mPref->SetBoolPref(prefName, PR_FALSE);
-  }
-
-  return rv;
-}
-
-nsresult
-nsNSSDialogs::ConfirmPostToInsecure(nsIInterfaceRequestor *ctx, PRBool* _result)
-{
-  nsresult rv;
-
-  rv = ConfirmDialog(ctx, INSECURE_SUBMIT_PREF,
-                     NS_LITERAL_STRING("PostToInsecureFromInsecureMessage").get(),
-                     NS_LITERAL_STRING("PostToInsecureFromInsecureShowAgain").get(),
-                     _result);
-
-  return rv;
-}
-
-nsresult
-nsNSSDialogs::ConfirmPostToInsecureFromSecure(nsIInterfaceRequestor *ctx, PRBool* _result)
-{
-  nsresult rv;
-
-  rv = ConfirmDialog(ctx, nsnull, // No preference for this one - it's too important
-                     NS_LITERAL_STRING("PostToInsecureFromSecureMessage").get(),
-                     nsnull, 
-                     _result);
-
-  return rv;
-}
-
-nsresult
-nsNSSDialogs::ConfirmDialog(nsIInterfaceRequestor *ctx, const char *prefName,
-                            const PRUnichar *messageName, 
-                            const PRUnichar *showAgainName, 
-                            PRBool* _result)
-{
-  nsresult rv;
-
-  // Get user's preference for this alert
-  // prefName, showAgainName are null if there is no preference for this dialog
-  PRBool prefValue = PR_TRUE;
-  
-  if (prefName != nsnull) {
-    rv = mPref->GetBoolPref(prefName, &prefValue);
-    if (NS_FAILED(rv)) prefValue = PR_TRUE;
-  }
-  
-  // Stop if confirm is not requested
-  if (!prefValue) {
-    *_result = PR_TRUE;
-    return NS_OK;
-  }
-  
-  // Get Prompt to use
-  nsCOMPtr<nsIPrompt> prompt = do_GetInterface(ctx);
-  if (!prompt) return NS_ERROR_FAILURE;
-
-  // Get messages strings from localization file
-  nsXPIDLString windowTitle, message, alertMe, cont;
-
-  mStringBundle->GetStringFromName(NS_LITERAL_STRING("Title").get(),
-                                   getter_Copies(windowTitle));
-  mStringBundle->GetStringFromName(messageName,
-                                   getter_Copies(message));
-  if (showAgainName != nsnull) {
-    mStringBundle->GetStringFromName(showAgainName,
-                                     getter_Copies(alertMe));
-  }
-  mStringBundle->GetStringFromName(NS_LITERAL_STRING("Continue").get(),
-                                   getter_Copies(cont));
-  // alertMe is allowed to be null
-  if (!windowTitle || !message || !cont) return NS_ERROR_FAILURE;
-      
-  // Replace # characters with newlines to lay out the dialog.
-  PRUnichar* msgchars = NS_CONST_CAST(PRUnichar*, message.get());
-  
-  PRUint32 i = 0;
-  for (i = 0; msgchars[i] != '\0'; i++) {
-    if (msgchars[i] == '#') {
-      msgchars[i] = '\n';
-    }
-  }  
-
-  PRInt32 buttonPressed;
-
-  rv  = prompt->ConfirmEx(windowTitle, 
-                          message, 
-                          (nsIPrompt::BUTTON_TITLE_IS_STRING * nsIPrompt::BUTTON_POS_0) +
-                          (nsIPrompt::BUTTON_TITLE_CANCEL * nsIPrompt::BUTTON_POS_1),
-                          cont,
-                          nsnull,
-                          nsnull,
-                          alertMe, 
-                          &prefValue, 
-                          &buttonPressed);
-
-  if (NS_FAILED(rv)) return rv;
-
-  *_result = (buttonPressed != 1);
-
-  if (!prefValue && prefName != nsnull) {
-    mPref->SetBoolPref(prefName, PR_FALSE);
-  }
-
-  return rv;
-}
-
-/* void downloadCACert (in nsIInterfaceRequestor ctx,
-                        in nsIX509Cert cert,
-                        out trust,
-                        out canceled); */
 NS_IMETHODIMP 
-nsNSSDialogs::DownloadCACert(nsIInterfaceRequestor *ctx, 
-                             nsIX509Cert *cert,
-                             PRUint32 *_trust,
-                             PRBool *_canceled)
+nsNSSDialogs::ConfirmDownloadCACert(nsIInterfaceRequestor *ctx, 
+                                    nsIX509Cert *cert,
+                                    PRUint32 *_trust,
+                                    PRBool *_retval)
 {
   nsresult rv;
 
-  *_canceled = PR_FALSE;
+  *_retval = PR_TRUE;
 
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
@@ -705,30 +426,27 @@ nsNSSDialogs::DownloadCACert(nsIInterfaceRequestor *ctx,
   *_trust |= (email) ? nsIX509CertDB::TRUSTED_EMAIL : 0;
   *_trust |= (objsign) ? nsIX509CertDB::TRUSTED_OBJSIGN : 0;
 
-  *_canceled = (status == 0)?PR_TRUE:PR_FALSE;
+  *_retval = (status == 0)?PR_FALSE:PR_TRUE;
 
   return rv;
 }
 
 
 NS_IMETHODIMP 
-nsNSSDialogs::CACertExists(nsIInterfaceRequestor *ctx,PRBool *_canceled)
-								
+nsNSSDialogs::NotifyCACertExists(nsIInterfaceRequestor *ctx)
 {
   nsresult rv;
 
-  *_canceled = PR_FALSE;
-
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
 
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
 
   
   rv = nsNSSDialogHelper::openDialog(parent, 
-                                   "chrome://pippki/content/cacertexists.xul",
-								    block);
+                                     "chrome://pippki/content/cacertexists.xul",
+                                     block);
 
   return rv;
 }
@@ -745,7 +463,7 @@ nsNSSDialogs::ChooseCertificate(nsIInterfaceRequestor *ctx, const PRUnichar *cn,
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
 
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
 
   block->SetNumberStrings(4+count*2);
@@ -760,13 +478,13 @@ nsNSSDialogs::ChooseCertificate(nsIInterfaceRequestor *ctx, const PRUnichar *cn,
   if (NS_FAILED(rv)) return rv;
 
   for (i = 0; i < count; i++) {
-	  rv = block->SetString(i+3, certNickList[i]);
-	  if (NS_FAILED(rv)) return rv;
+    rv = block->SetString(i+3, certNickList[i]);
+    if (NS_FAILED(rv)) return rv;
   }
 
   for (i = 0; i < count; i++) {
-	  rv = block->SetString(i+count+3, certDetailsList[i]);
-	  if (NS_FAILED(rv)) return rv;
+    rv = block->SetString(i+count+3, certDetailsList[i]);
+    if (NS_FAILED(rv)) return rv;
   }
 
   rv = block->SetInt(0, count);
@@ -807,19 +525,19 @@ nsNSSDialogs::PickCertificate(nsIInterfaceRequestor *ctx,
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
 
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
 
   block->SetNumberStrings(1+count*2);
 
   for (i = 0; i < count; i++) {
-	  rv = block->SetString(i, certNickList[i]);
-	  if (NS_FAILED(rv)) return rv;
+    rv = block->SetString(i, certNickList[i]);
+    if (NS_FAILED(rv)) return rv;
   }
 
   for (i = 0; i < count; i++) {
-	  rv = block->SetString(i+count, certDetailsList[i]);
-	  if (NS_FAILED(rv)) return rv;
+    rv = block->SetString(i+count, certDetailsList[i]);
+    if (NS_FAILED(rv)) return rv;
   }
 
   rv = block->SetInt(0, count);
@@ -846,21 +564,16 @@ nsNSSDialogs::PickCertificate(nsIInterfaceRequestor *ctx,
 }
 
 
-/*
- * void setPKCS12FilePassword(in nsIInterfaceRequestor ctx, 
- *                            out wstring password,
- *                            out boolean canceled);
- */
 NS_IMETHODIMP 
 nsNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx, 
-                                    PRUnichar **_password,
-                                    PRBool *_canceled)
+                                    nsAString &_password,
+                                    PRBool *_retval)
 {
   nsresult rv;
-  *_canceled = PR_FALSE;
+  *_retval = PR_TRUE;
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
   // open up the window
   rv = nsNSSDialogHelper::openDialog(parent,
@@ -871,29 +584,29 @@ nsNSSDialogs::SetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
   PRInt32 status;
   rv = block->GetInt(1, &status);
   if (NS_FAILED(rv)) return rv;
-  *_canceled = (status == 0) ? PR_TRUE : PR_FALSE;
-  if (!*_canceled) {
+  *_retval = (status == 0) ? PR_FALSE : PR_TRUE;
+  if (*_retval) {
     // retrieve the password
-    rv = block->GetString(2, _password);
+    PRUnichar *pw;
+    rv = block->GetString(2, &pw);
+    if (NS_SUCCEEDED(rv)) {
+      _password = pw;
+      nsMemory::Free(pw);
+    }
   }
   return rv;
 }
 
-/*
- *  void getPKCS12FilePassword(in nsIInterfaceRequestor ctx, 
- *                             out wstring password,
- *                             out boolean canceled);
- */
 NS_IMETHODIMP 
 nsNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx, 
-                                    PRUnichar **_password,
-                                    PRBool *_canceled)
+                                    nsAString &_password,
+                                    PRBool *_retval)
 {
   nsresult rv;
-  *_canceled = PR_FALSE;
+  *_retval = PR_TRUE;
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
   // open up the window
   rv = nsNSSDialogHelper::openDialog(parent,
@@ -904,17 +617,23 @@ nsNSSDialogs::GetPKCS12FilePassword(nsIInterfaceRequestor *ctx,
   PRInt32 status;
   rv = block->GetInt(1, &status);
   if (NS_FAILED(rv)) return rv;
-  *_canceled = (status == 0) ? PR_TRUE : PR_FALSE;
-  if (!*_canceled) {
+  *_retval = (status == 0) ? PR_FALSE : PR_TRUE;
+  if (*_retval) {
     // retrieve the password
-    rv = block->GetString(2, _password);
+    PRUnichar *pw;
+    rv = block->GetString(2, &pw);
+    if (NS_SUCCEEDED(rv)) {
+      _password = pw;
+      nsMemory::Free(pw);
+    }
   }
   return rv;
 }
 
 /* void viewCert (in nsIX509Cert cert); */
 NS_IMETHODIMP 
-nsNSSDialogs::ViewCert(nsIX509Cert *cert)
+nsNSSDialogs::ViewCert(nsIInterfaceRequestor *ctx, 
+                       nsIX509Cert *cert)
 {
   nsresult rv;
 
@@ -926,7 +645,10 @@ nsNSSDialogs::ViewCert(nsIX509Cert *cert)
   if (NS_FAILED(rv))
     return rv;
 
-  rv = nsNSSDialogHelper::openDialog(nsnull,
+  // Get the parent window for the dialog
+  nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(ctx);
+
+  rv = nsNSSDialogHelper::openDialog(parent,
                                      "chrome://pippki/content/certViewer.xul",
                                      block);
   return rv;
@@ -956,14 +678,14 @@ nsNSSDialogs::ChooseToken(nsIInterfaceRequestor *aCtx, const PRUnichar **aTokenL
   // Get the parent window for the dialog
   nsCOMPtr<nsIDOMWindowInternal> parent = do_GetInterface(aCtx);
 
-  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance("@mozilla.org/embedcomp/dialogparam;1"));
+  nsCOMPtr<nsIDialogParamBlock> block(do_CreateInstance(NS_DIALOGPARAMBLOCK_CONTRACTID));
   if (!block) return NS_ERROR_FAILURE;
 
   block->SetNumberStrings(aCount);
 
   for (i = 0; i < aCount; i++) {
-	  rv = block->SetString(i, aTokenList[i]);
-	  if (NS_FAILED(rv)) return rv;
+    rv = block->SetString(i, aTokenList[i]);
+    if (NS_FAILED(rv)) return rv;
   }
 
   rv = block->SetInt(0, aCount);
@@ -1020,3 +742,5 @@ nsNSSDialogs::ConfirmKeyEscrow(nsIX509Cert *escrowAuthority, PRBool *_retval)
   } 
   return rv;
 }
+
+
