@@ -32,7 +32,7 @@
  */
 
 #ifdef DEBUG
-static const char CVS_ID[] = "@(#) $RCSfile: tdcache.c,v $ $Revision: 1.26.4.1 $ $Date: 2002/03/08 22:18:44 $ $Name:  $";
+static const char CVS_ID[] = "@(#) $RCSfile: tdcache.c,v $ $Revision: 1.38 $ $Date: 2003/12/19 22:33:12 $ $Name: MOZILLA_1_7_6_RELEASE $";
 #endif /* DEBUG */
 
 #ifndef PKIM_H
@@ -61,6 +61,8 @@ static const char CVS_ID[] = "@(#) $RCSfile: tdcache.c,v $ $Revision: 1.26.4.1 $
 
 #ifdef NSS_3_4_CODE
 #include "cert.h"
+#include "dev.h"
+#include "pki3hack.h"
 #endif
 
 #ifdef DEBUG_CACHE
@@ -124,6 +126,7 @@ struct cache_entry_str
     PRUint32 hits;
     PRTime lastHit;
     NSSArena *arena;
+    NSSUTF8 *nickname;
 };
 
 typedef struct cache_entry_str cache_entry;
@@ -139,6 +142,7 @@ new_cache_entry(NSSArena *arena, void *value, PRBool ownArena)
 	if (ownArena) {
 	    ce->arena = arena;
 	}
+	ce->nickname = NULL;
     }
     return ce;
 }
@@ -161,8 +165,7 @@ static PRIntn subject_list_sort(void *v1, void *v2)
  * types/functions static
  */
 NSS_IMPLEMENT PRStatus
-nssTrustDomain_InitializeCache
-(
+nssTrustDomain_InitializeCache (
   NSSTrustDomain *td,
   PRUint32 cacheSize
 )
@@ -231,13 +234,20 @@ loser:
  * clean shutdown, it is necessary for there to be no certs in the cache.
  */
 
+extern const NSSError NSS_ERROR_INTERNAL_ERROR;
+extern const NSSError NSS_ERROR_BUSY;
+
 NSS_IMPLEMENT PRStatus
-nssTrustDomain_DestroyCache
-(
+nssTrustDomain_DestroyCache (
   NSSTrustDomain *td
 )
 {
     if (!td->cache) {
+	nss_SetError(NSS_ERROR_INTERNAL_ERROR);
+	return PR_FAILURE;
+    }
+    if (nssHash_Count(td->cache->issuerAndSN) > 0) {
+	nss_SetError(NSS_ERROR_BUSY);
 	return PR_FAILURE;
     }
     PZ_DestroyLock(td->cache->lock);
@@ -254,8 +264,7 @@ nssTrustDomain_DestroyCache
 }
 
 static PRStatus
-remove_issuer_and_serial_entry
-(
+remove_issuer_and_serial_entry (
   nssTDCertificateCache *cache,
   NSSCertificate *cert
 )
@@ -269,11 +278,11 @@ remove_issuer_and_serial_entry
 }
 
 static PRStatus
-remove_subject_entry
-(
+remove_subject_entry (
   nssTDCertificateCache *cache,
   NSSCertificate *cert,
   nssList **subjectList,
+  NSSUTF8 **nickname,
   NSSArena **arena
 )
 {
@@ -287,6 +296,7 @@ remove_subject_entry
 	/* Remove the cert from the subject hash */
 	nssList_Remove(ce->entry.list, cert);
 	*subjectList = ce->entry.list;
+	*nickname = ce->nickname;
 	*arena = ce->arena;
 	nssrv = PR_SUCCESS;
 #ifdef DEBUG_CACHE
@@ -300,15 +310,13 @@ remove_subject_entry
 }
 
 static PRStatus
-remove_nickname_entry
-(
+remove_nickname_entry (
   nssTDCertificateCache *cache,
-  NSSCertificate *cert,
+  NSSUTF8 *nickname,
   nssList *subjectList
 )
 {
     PRStatus nssrv;
-    NSSUTF8 *nickname = NSSCertificate_GetNickname(cert, NULL);
     if (nickname) {
 	nssHash_Remove(cache->nickname, nickname);
 	nssrv = PR_SUCCESS;
@@ -322,8 +330,7 @@ remove_nickname_entry
 }
 
 static PRStatus
-remove_email_entry
-(
+remove_email_entry (
   nssTDCertificateCache *cache,
   NSSCertificate *cert,
   nssList *subjectList
@@ -363,49 +370,35 @@ remove_email_entry
 }
 
 NSS_IMPLEMENT void
-nssTrustDomain_RemoveCertFromCache
-(
+nssTrustDomain_RemoveCertFromCacheLOCKED (
   NSSTrustDomain *td,
   NSSCertificate *cert
 )
 {
     nssList *subjectList;
-    PRStatus nssrv;
     cache_entry *ce;
     NSSArena *arena;
+    NSSUTF8 *nickname;
+
 #ifdef DEBUG_CACHE
     log_cert_ref("attempt to remove cert", cert);
 #endif
-    PZ_Lock(td->cache->lock);
     ce = (cache_entry *)nssHash_Lookup(td->cache->issuerAndSN, cert);
     if (!ce || ce->entry.cert != cert) {
 	/* If it's not in the cache, or a different cert is (this is really
 	 * for safety reasons, though it shouldn't happen), do nothing 
 	 */
-	PZ_Unlock(td->cache->lock);
 #ifdef DEBUG_CACHE
 	PR_LOG(s_log, PR_LOG_DEBUG, ("but it wasn't in the cache"));
 #endif
 	return;
     }
-    nssrv = remove_issuer_and_serial_entry(td->cache, cert);
-    if (nssrv != PR_SUCCESS) {
-	goto loser;
-    }
-    nssrv = remove_subject_entry(td->cache, cert, &subjectList, &arena);
-    if (nssrv != PR_SUCCESS) {
-	goto loser;
-    }
+    (void)remove_issuer_and_serial_entry(td->cache, cert);
+    (void)remove_subject_entry(td->cache, cert, &subjectList, 
+                               &nickname, &arena);
     if (nssList_Count(subjectList) == 0) {
-	PRStatus nssrv2;
-	nssrv = remove_nickname_entry(td->cache, cert, subjectList);
-	nssrv2 = remove_email_entry(td->cache, cert, subjectList);
-#ifndef NSS_3_4_CODE
-	/* XXX Again, 3.4 allows for certs w/o either nickname or email */
-	if (nssrv != PR_SUCCESS && nssrv2 != PR_SUCCESS) {
-	    goto loser;
-	}
-#endif
+	(void)remove_nickname_entry(td->cache, nickname, subjectList);
+	(void)remove_email_entry(td->cache, cert, subjectList);
 	(void)nssList_Destroy(subjectList);
 	nssHash_Remove(td->cache->subject, &cert->subject);
 	/* there are no entries left for this subject, free the space used
@@ -415,49 +408,57 @@ nssTrustDomain_RemoveCertFromCache
 	    nssArena_Destroy(arena);
 	}
     }
-    PZ_Unlock(td->cache->lock);
-    NSSCertificate_Destroy(cert); /* release the reference */
-    return;
-loser:
-    /* if here, then the cache is inconsistent.  For now, flush it. */
-    PZ_Unlock(td->cache->lock);
-#ifdef DEBUG_CACHE
-    PR_LOG(s_log, PR_LOG_DEBUG, ("remove cert failed, flushing"));
-#endif
-    nssTrustDomain_FlushCache(td, -1.0);
-    NSSCertificate_Destroy(cert); /* release the reference */
 }
 
-/* This is used to remove all certs below a certain threshold, where
- * the value is determined by how many times the cert has been requested
- * and when the last request was.
- */
 NSS_IMPLEMENT void
-nssTrustDomain_FlushCache
-(
-  NSSTrustDomain *td,
-  PRFloat64 threshold
+nssTrustDomain_LockCertCache (
+  NSSTrustDomain *td
 )
 {
+    PZ_Lock(td->cache->lock);
 }
 
-struct token_cert_destructor {
-    nssTDCertificateCache *cache;
+NSS_IMPLEMENT void
+nssTrustDomain_UnlockCertCache (
+  NSSTrustDomain *td
+)
+{
+    PZ_Unlock(td->cache->lock);
+}
+
+struct token_cert_dtor {
     NSSToken *token;
+    nssTDCertificateCache *cache;
+    NSSCertificate **certs;
+    PRUint32 numCerts, arrSize;
 };
 
 static void 
 remove_token_certs(const void *k, void *v, void *a)
 {
-#if 0
-    struct NSSItem *identifier = (struct NSSItem *)k;
-    NSSCertificate *c = (NSSCertificate *)v;
-    struct token_cert_destructor *tcd = (struct token_cert_destructor *)a;
-    if (c->token == tcd->token) {
-	nssHash_Remove(tcd->cache->issuerAndSN, identifier);
-	/* remove from the other hashes */
+    NSSCertificate *c = (NSSCertificate *)k;
+    nssPKIObject *object = &c->object;
+    struct token_cert_dtor *dtor = a;
+    PRUint32 i;
+    PZ_Lock(object->lock);
+    for (i=0; i<object->numInstances; i++) {
+	if (object->instances[i]->token == dtor->token) {
+	    nssCryptokiObject_Destroy(object->instances[i]);
+	    object->instances[i] = object->instances[object->numInstances-1];
+	    object->instances[object->numInstances-1] = NULL;
+	    object->numInstances--;
+	    dtor->certs[dtor->numCerts++] = c;
+	    if (dtor->numCerts == dtor->arrSize) {
+		dtor->arrSize *= 2;
+		dtor->certs = nss_ZREALLOCARRAY(dtor->certs, 
+		                                NSSCertificate *,
+		                                dtor->arrSize);
+	    }
+	    break;
+	}
     }
-#endif
+    PZ_Unlock(object->lock);
+    return;
 }
 
 /* 
@@ -465,24 +466,84 @@ remove_token_certs(const void *k, void *v, void *a)
  * needed if the token is removed. 
  */
 NSS_IMPLEMENT PRStatus
-nssTrustDomain_RemoveTokenCertsFromCache
-(
+nssTrustDomain_RemoveTokenCertsFromCache (
   NSSTrustDomain *td,
   NSSToken *token
 )
 {
-    struct token_cert_destructor tcd;
-    tcd.cache = td->cache;
-    tcd.token = token;
+    NSSCertificate **certs;
+    PRUint32 i, arrSize = 10;
+    struct token_cert_dtor dtor;
+    certs = nss_ZNEWARRAY(NULL, NSSCertificate *, arrSize);
+    if (!certs) {
+	return PR_FAILURE;
+    }
+    dtor.cache = td->cache;
+    dtor.token = token;
+    dtor.certs = certs;
+    dtor.numCerts = 0;
+    dtor.arrSize = arrSize;
     PZ_Lock(td->cache->lock);
-    nssHash_Iterate(td->cache->issuerAndSN, remove_token_certs, (void *)&tcd);
+    nssHash_Iterate(td->cache->issuerAndSN, remove_token_certs, (void *)&dtor);
+    for (i=0; i<dtor.numCerts; i++) {
+	if (dtor.certs[i]->object.numInstances == 0) {
+	    nssTrustDomain_RemoveCertFromCacheLOCKED(td, dtor.certs[i]);
+	    dtor.certs[i] = NULL;  /* skip this cert in the second for loop */
+	}
+    }
     PZ_Unlock(td->cache->lock);
+    for (i=0; i<dtor.numCerts; i++) {
+	if (dtor.certs[i]) {
+	    STAN_ForceCERTCertificateUpdate(dtor.certs[i]);
+	}
+    }
+    nss_ZFreeIf(dtor.certs);
+    return PR_SUCCESS;
+}
+
+NSS_IMPLEMENT PRStatus
+nssTrustDomain_UpdateCachedTokenCerts (
+  NSSTrustDomain *td,
+  NSSToken *token
+)
+{
+    NSSCertificate **cp, **cached = NULL;
+    nssList *certList;
+    PRUint32 count;
+    certList = nssList_Create(NULL, PR_FALSE);
+    if (!certList) return PR_FAILURE;
+    (void *)nssTrustDomain_GetCertsFromCache(td, certList);
+    count = nssList_Count(certList);
+    if (count > 0) {
+	cached = nss_ZNEWARRAY(NULL, NSSCertificate *, count + 1);
+	if (!cached) {
+	    return PR_FAILURE;
+	}
+	nssList_GetArray(certList, (void **)cached, count);
+	nssList_Destroy(certList);
+	for (cp = cached; *cp; cp++) {
+	    nssCryptokiObject *instance;
+	    NSSCertificate *c = *cp;
+	    nssTokenSearchType tokenOnly = nssTokenSearchType_TokenOnly;
+	    instance = nssToken_FindCertificateByIssuerAndSerialNumber(
+	                                                       token,
+                                                               NULL,
+                                                               &c->issuer,
+                                                               &c->serial,
+                                                               tokenOnly,
+                                                               NULL);
+	    if (instance) {
+		nssPKIObject_AddInstance(&c->object, instance);
+		STAN_ForceCERTCertificateUpdate(c);
+	    }
+	}
+	nssCertificateArray_Destroy(cached);
+    }
     return PR_SUCCESS;
 }
 
 static PRStatus
-add_issuer_and_serial_entry
-(
+add_issuer_and_serial_entry (
   NSSArena *arena,
   nssTDCertificateCache *cache, 
   NSSCertificate *cert
@@ -497,11 +558,11 @@ add_issuer_and_serial_entry
 }
 
 static PRStatus
-add_subject_entry
-(
+add_subject_entry (
   NSSArena *arena,
   nssTDCertificateCache *cache, 
   NSSCertificate *cert,
+  NSSUTF8 *nickname,
   nssList **subjectList
 )
 {
@@ -529,6 +590,9 @@ add_subject_entry
 	if (!ce) {
 	    return PR_FAILURE;
 	}
+	if (nickname) {
+	    ce->nickname = nssUTF8_Duplicate(nickname, arena);
+	}
 	nssList_SetSortFunction(list, subject_list_sort);
 	/* Add the cert entry to this list of subjects */
 	nssrv = nssList_AddUnique(list, cert);
@@ -553,16 +617,14 @@ add_subject_entry
 }
 
 static PRStatus
-add_nickname_entry
-(
+add_nickname_entry (
   NSSArena *arena,
   nssTDCertificateCache *cache, 
-  NSSCertificate *cert,
+  NSSUTF8 *certNickname,
   nssList *subjectList
 )
 {
     PRStatus nssrv = PR_SUCCESS;
-    NSSUTF8 *certNickname = NSSCertificate_GetNickname(cert, NULL);
     cache_entry *ce;
     ce = (cache_entry *)nssHash_Lookup(cache->nickname, certNickname);
     if (ce) {
@@ -590,8 +652,7 @@ add_nickname_entry
 }
 
 static PRStatus
-add_email_entry
-(
+add_email_entry (
   nssTDCertificateCache *cache, 
   NSSCertificate *cert,
   nssList *subjectList
@@ -654,29 +715,87 @@ add_email_entry
 
 extern const NSSError NSS_ERROR_CERTIFICATE_IN_CACHE;
 
-static PRStatus
-add_cert_to_cache
-(
+static void
+remove_object_instances (
+  nssPKIObject *object,
+  nssCryptokiObject **instances,
+  int numInstances
+)
+{
+    int i;
+
+    for (i = 0; i < numInstances; i++) {
+	nssPKIObject_RemoveInstanceForToken(object, instances[i]->token);
+    }
+}
+
+static SECStatus
+merge_object_instances (
+  nssPKIObject *to,
+  nssPKIObject *from
+)
+{
+    nssCryptokiObject **instances, **ci;
+    int i;
+    SECStatus rv = SECSuccess;
+
+    instances = nssPKIObject_GetInstances(from);
+    if (instances == NULL) {
+	return SECFailure;
+    }
+    for (ci = instances, i = 0; *ci; ci++, i++) {
+	nssCryptokiObject *instance = nssCryptokiObject_Clone(*ci);
+	if (instance) {
+	    if (nssPKIObject_AddInstance(to, instance) == SECSuccess) {
+		continue;
+	    }
+	    nssCryptokiObject_Destroy(instance);
+	}
+	remove_object_instances(to, instances, i);
+	rv = SECFailure;
+	break;
+    }
+    nssCryptokiObjectArray_Destroy(instances);
+    return rv;
+}
+
+static NSSCertificate *
+add_cert_to_cache (
   NSSTrustDomain *td, 
   NSSCertificate *cert
 )
 {
     NSSArena *arena = NULL;
-    nssList *subjectList;
+    nssList *subjectList = NULL;
     PRStatus nssrv;
     PRUint32 added = 0;
+    cache_entry *ce;
+    NSSCertificate *rvCert = NULL;
+    NSSUTF8 *certNickname = nssCertificate_GetNickname(cert, NULL);
+
     PZ_Lock(td->cache->lock);
     /* If it exists in the issuer/serial hash, it's already in all */
-    if (nssHash_Exists(td->cache->issuerAndSN, cert)) {
+    ce = (cache_entry *)nssHash_Lookup(td->cache->issuerAndSN, cert);
+    if (ce) {
+	ce->hits++;
+	ce->lastHit = PR_Now();
+	rvCert = nssCertificate_AddRef(ce->entry.cert);
 #ifdef DEBUG_CACHE
 	log_cert_ref("attempted to add cert already in cache", cert);
 #endif
 	PZ_Unlock(td->cache->lock);
-	/* collision - most likely, somebody else already added the cert
+	/* collision - somebody else already added the cert
 	 * to the cache before this thread got around to it.
 	 */
-	nss_SetError(NSS_ERROR_CERTIFICATE_IN_CACHE);
-	return PR_FAILURE;
+	/* merge the instances of the cert */
+	if (merge_object_instances(&rvCert->object, &cert->object)
+							!= SECSuccess) {
+	    nssCertificate_Destroy(rvCert);
+	    return NULL;
+	}
+	STAN_ForceCERTCertificateUpdate(rvCert);
+	nssCertificate_Destroy(cert);
+	return rvCert;
     }
     /* create a new cache entry for this cert within the cert's arena*/
     nssrv = add_issuer_and_serial_entry(cert->object.arena, td->cache, cert);
@@ -690,7 +809,8 @@ add_cert_to_cache
 	goto loser;
     }
     /* create a new subject list for this cert, or add to existing */
-    nssrv = add_subject_entry(arena, td->cache, cert, &subjectList);
+    nssrv = add_subject_entry(arena, td->cache, cert, 
+						certNickname, &subjectList);
     if (nssrv != PR_SUCCESS) {
 	goto loser;
     }
@@ -698,9 +818,9 @@ add_cert_to_cache
     /* If a new subject entry was created, also need nickname and/or email */
     if (subjectList != NULL) {
 	PRBool handle = PR_FALSE;
-	NSSUTF8 *certNickname = NSSCertificate_GetNickname(cert, NULL);
 	if (certNickname) {
-	    nssrv = add_nickname_entry(arena, td->cache, cert, subjectList);
+	    nssrv = add_nickname_entry(arena, td->cache, 
+						certNickname, subjectList);
 	    if (nssrv != PR_SUCCESS) {
 		goto loser;
 	    }
@@ -727,9 +847,9 @@ add_cert_to_cache
 	}
 #endif
     }
-    nssCertificate_AddRef(cert);
+    rvCert = cert;
     PZ_Unlock(td->cache->lock);
-    return nssrv;
+    return rvCert;
 loser:
     /* Remove any handles that have been created */
     subjectList = NULL;
@@ -737,10 +857,11 @@ loser:
 	(void)remove_issuer_and_serial_entry(td->cache, cert);
     }
     if (added >= 2) {
-	(void)remove_subject_entry(td->cache, cert, &subjectList, &arena);
+	(void)remove_subject_entry(td->cache, cert, &subjectList, 
+						&certNickname, &arena);
     }
     if (added == 3 || added == 5) {
-	(void)remove_nickname_entry(td->cache, cert, subjectList);
+	(void)remove_nickname_entry(td->cache, certNickname, subjectList);
     }
     if (added >= 4) {
 	(void)remove_email_entry(td->cache, cert, subjectList);
@@ -753,51 +874,31 @@ loser:
 	nssArena_Destroy(arena);
     }
     PZ_Unlock(td->cache->lock);
-    return PR_FAILURE;
+    return NULL;
 }
 
 NSS_IMPLEMENT PRStatus
-nssTrustDomain_AddCertsToCache
-(
+nssTrustDomain_AddCertsToCache (
   NSSTrustDomain *td,
   NSSCertificate **certs,
   PRUint32 numCerts
 )
 {
     PRUint32 i;
-    NSSError e;
+    NSSCertificate *c;
     for (i=0; i<numCerts && certs[i]; i++) {
-	if (add_cert_to_cache(td, certs[i]) != PR_SUCCESS) {
-	    if ((e = NSS_GetError()) == NSS_ERROR_CERTIFICATE_IN_CACHE) {
-		/* collision - delete and replace the cert here in favor
-		 * of the already cached entry.  This is safe as long as
-		 * the cert being added here only has a single reference.
-		 * This should be the case as this function is only called
-		 * immediately following a traversal and before any certs
-		 * are returned to the caller.
-		 */
-		NSSCertificate *c;
-		c = nssTrustDomain_GetCertForIssuerAndSNFromCache(td,
-		                                            &certs[i]->issuer,
-		                                            &certs[i]->serial);
-		if (c != certs[i]) {
-		    NSSCertificate_Destroy(certs[i]);
-		    certs[i] = c;
-		} else {
-		    NSSCertificate_Destroy(c);
-		}
-		nss_ClearErrorStack();
-		continue;
-	    }
+	c = add_cert_to_cache(td, certs[i]);
+	if (c == NULL) {
 	    return PR_FAILURE;
-	} 
+	} else {
+	    certs[i] = c;
+	}
     }
     return PR_SUCCESS;
 }
 
 static NSSCertificate **
-collect_subject_certs
-(
+collect_subject_certs (
   nssList *subjectList,
   nssList *rvCertListOpt
 )
@@ -830,8 +931,7 @@ collect_subject_certs
  * Find all cached certs with this subject.
  */
 NSS_IMPLEMENT NSSCertificate **
-nssTrustDomain_GetCertsForSubjectFromCache
-(
+nssTrustDomain_GetCertsForSubjectFromCache (
   NSSTrustDomain *td,
   NSSDER *subject,
   nssList *certListOpt
@@ -860,8 +960,7 @@ nssTrustDomain_GetCertsForSubjectFromCache
  * Find all cached certs with this label.
  */
 NSS_IMPLEMENT NSSCertificate **
-nssTrustDomain_GetCertsForNicknameFromCache
-(
+nssTrustDomain_GetCertsForNicknameFromCache (
   NSSTrustDomain *td,
   NSSUTF8 *nickname,
   nssList *certListOpt
@@ -890,8 +989,7 @@ nssTrustDomain_GetCertsForNicknameFromCache
  * Find all cached certs with this email address.
  */
 NSS_IMPLEMENT NSSCertificate **
-nssTrustDomain_GetCertsForEmailAddressFromCache
-(
+nssTrustDomain_GetCertsForEmailAddressFromCache (
   NSSTrustDomain *td,
   NSSASCII7 *email,
   nssList *certListOpt
@@ -955,8 +1053,7 @@ nssTrustDomain_GetCertsForEmailAddressFromCache
  * Look for a specific cert in the cache
  */
 NSS_IMPLEMENT NSSCertificate *
-nssTrustDomain_GetCertForIssuerAndSNFromCache
-(
+nssTrustDomain_GetCertForIssuerAndSNFromCache (
   NSSTrustDomain *td,
   NSSDER *issuer,
   NSSDER *serial
@@ -989,8 +1086,7 @@ nssTrustDomain_GetCertForIssuerAndSNFromCache
 
 #ifdef NSS_3_4_CODE
 static PRStatus
-issuer_and_serial_from_encoding
-(
+issuer_and_serial_from_encoding (
   NSSBER *encoding, 
   NSSDER *issuer, 
   NSSDER *serial
@@ -1020,8 +1116,7 @@ issuer_and_serial_from_encoding
  * Look for a specific cert in the cache
  */
 NSS_IMPLEMENT NSSCertificate *
-nssTrustDomain_GetCertByDERFromCache
-(
+nssTrustDomain_GetCertByDERFromCache (
   NSSTrustDomain *td,
   NSSDER *der
 )
@@ -1055,8 +1150,7 @@ static void cert_iter(const void *k, void *v, void *a)
 }
 
 NSS_EXTERN NSSCertificate **
-nssTrustDomain_GetCertsFromCache
-(
+nssTrustDomain_GetCertsFromCache (
   NSSTrustDomain *td,
   nssList *certListOpt
 )
@@ -1082,8 +1176,7 @@ nssTrustDomain_GetCertsFromCache
 }
 
 NSS_IMPLEMENT void
-nssTrustDomain_DumpCacheInfo
-(
+nssTrustDomain_DumpCacheInfo (
   NSSTrustDomain *td,
   void (* cert_dump_iter)(const void *, void *, void *),
   void *arg
