@@ -61,7 +61,7 @@
 #include "nsplugindefs.h"
 #include "nsMacEventHandler.h"
 #include "nsMacResources.h"
-#include "nsRegionMac.h"
+#include "nsIRegion.h"
 #include "nsIRollupListener.h"
 
 #include "nsCarbonHelpers.h"
@@ -84,6 +84,10 @@ nsIWidget         * gRollupWidget   = nsnull;
 static NMRec	gNMRec;
 static Boolean	gNotificationInstalled = false;
 
+//used to animate the Arrow + Beachball cursor
+static CursorSpinner *gCursorSpinner = nsnull;
+static const int kSpinCursorFirstFrame = 200;
+
 // Routines for iterating over the rects of a region. Carbon and pre-Carbon
 // do this differently so provide a way to do both.
 #if TARGET_CARBON
@@ -105,13 +109,14 @@ static void blinkRect(const Rect* r, PRBool isPaint);
 static void blinkRgn(RgnHandle rgn, PRBool isPaint);
 #endif
 
-
+#if defined(INVALIDATE_DEBUGGING) || defined(PAINT_DEBUGGING) || defined (PINK_PROFILING)
 static Boolean KeyDown(const UInt8 theKey)
 {
 	KeyMap map;
 	GetKeys(map);
 	return ((*((UInt8 *)map + (theKey >> 3)) >> (theKey & 7)) & 1) != 0;
 }
+#endif
 
 #if defined(INVALIDATE_DEBUGGING) || defined(PAINT_DEBUGGING)
 
@@ -337,7 +342,7 @@ nsWindow::~nsWindow()
 	}
 }
 
-NS_IMPL_ISUPPORTS_INHERITED1(nsWindow, nsBaseWidget, nsIKBStateControl);
+NS_IMPL_ISUPPORTS_INHERITED2(nsWindow, nsBaseWidget, nsIKBStateControl, nsIPluginWidget)
 
 //-------------------------------------------------------------------------
 //
@@ -449,6 +454,7 @@ NS_IMETHODIMP nsWindow::Destroy()
 }
 
 #pragma mark -
+
 //-------------------------------------------------------------------------
 //
 // Get this nsWindow parent
@@ -687,6 +693,18 @@ nsIMenuBar* nsWindow::GetMenuBar()
   return mMenuBar;
 }
 
+PRBool OnJaguarOrLater() // Return true if we are on Mac OS X 10.2 or later
+{
+    static PRBool gInitVer = PR_FALSE;
+    static PRBool gOnJaguarOrLater = PR_FALSE;
+    if(!gInitVer)
+    {
+        gOnJaguarOrLater =
+            (nsToolkit::OSXVersion() >= MAC_OS_X_VERSION_10_2_HEX);
+        gInitVer = PR_TRUE;
+    }
+    return gOnJaguarOrLater;
+}
 
 //
 // SetCursor
@@ -697,125 +715,149 @@ NS_METHOD nsWindow::SetCursor(nsCursor aCursor)
 {
   nsBaseWidget::SetCursor(aCursor);
 	
-	// allow the cursor to be set internally if we're in the bg, but
-	// don't actually set it.
+  // allow the cursor to be set internally if we're in the bg, but
+  // don't actually set it.
   if ( !nsToolkit::IsAppInForeground() )
-    return NS_OK;
-
-  // mac specific cursor manipulation
-	if (nsToolkit::HasAppearanceManager())
-	{
-		short cursor = -1;
-	  switch (aCursor)
-	  {
-	    case eCursor_standard:				cursor = kThemeArrowCursor;						break;
-	    case eCursor_wait:						cursor = kThemeWatchCursor;						break;
-	    case eCursor_select:					cursor = kThemeIBeamCursor;						break;
-	    case eCursor_hyperlink:				cursor = kThemePointingHandCursor;		break;
-			case eCursor_sizeWE:					cursor = kThemeResizeLeftRightCursor;	break;
-			case eCursor_sizeNS:					cursor = 129; 	break;
-			case eCursor_sizeNW:					cursor = 130; 	break;
-			case eCursor_sizeSE:					cursor = 131; 	break;
-			case eCursor_sizeNE:					cursor = 132; 	break;
-			case eCursor_sizeSW:					cursor = 133; 	break;
-			case eCursor_arrow_north:			cursor = 134; 	break;
-			case eCursor_arrow_north_plus:cursor = 135; 	break;
-			case eCursor_arrow_south:			cursor = 136; 	break;
-			case eCursor_arrow_south_plus:cursor = 137; 	break;
-			case eCursor_arrow_west:			cursor = 138; 	break;
-			case eCursor_arrow_west_plus:	cursor = 139; 	break;
-			case eCursor_arrow_east:			cursor = 140; 	break;
-			case eCursor_arrow_east_plus:	cursor = 141; 	break;
-			case eCursor_crosshair:				cursor = kThemeCrossCursor; 					break;
-			case eCursor_move:						cursor = kThemeOpenHandCursor; 				break;
-			case eCursor_help:						cursor = 143; 	break;
-			case eCursor_copy:						cursor = 144; 	break; // CSS3
-			case eCursor_alias:						cursor = 145; 	break;
-			case eCursor_context_menu:		cursor = 146; 	break;
-			case eCursor_cell:						cursor = kThemePlusCursor; 			 			break;
-			case eCursor_grab:						cursor = kThemeOpenHandCursor; 				break;
-			case eCursor_grabbing:				cursor = kThemeClosedHandCursor; 			break;
-			case eCursor_spinning:				cursor = 149; break;  // better than kThemeSpinningCursor
-			case eCursor_count_up:				cursor = kThemeCountingUpHandCursor; 					break;
-			case eCursor_count_down:			cursor = kThemeCountingDownHandCursor;				break;
-			case eCursor_count_up_down:		cursor = kThemeCountingUpAndDownHandCursor; 	break;
-
-	  }
-	  if (cursor >= 0)
-	  {
-	  	if (cursor >= 128)
-	  	{
-        nsMacResources::OpenLocalResourceFile();
-        CursHandle cursHandle = ::GetCursor(cursor);
-        NS_ASSERTION ( cursHandle, "Can't load cursor, is the resource file installed correctly?" );
-        if ( cursHandle )
-          ::SetCursor(*cursHandle);
-        nsMacResources::CloseLocalResourceFile();
-			}
-			else
-	  		::SetThemeCursor(cursor);
-	  }
-  }
-  else
   {
-		short cursor = -1;
-	  switch (aCursor)
-	  {
-	    case eCursor_standard:				::InitCursor();					break;
-	    case eCursor_wait:						cursor = watchCursor;		break;
-	    case eCursor_select:					cursor = iBeamCursor;		break;
-	    case eCursor_hyperlink:				cursor = plusCursor;		break;
-			case eCursor_sizeWE:					cursor = 128; 					break;
-			case eCursor_sizeNS:					cursor = 129; 	break;
-			case eCursor_sizeNW:					cursor = 130; 	break;
-			case eCursor_sizeSE:					cursor = 131; 	break;
-			case eCursor_sizeNE:					cursor = 132; 	break;
-			case eCursor_sizeSW:					cursor = 133; 	break;
-			case eCursor_arrow_north:			cursor = 134; 	break;
-			case eCursor_arrow_north_plus:cursor = 135; 	break;
-			case eCursor_arrow_south:			cursor = 136; 	break;
-			case eCursor_arrow_south_plus:cursor = 137; 	break;
-			case eCursor_arrow_west:			cursor = 138; 	break;
-			case eCursor_arrow_west_plus:	cursor = 139; 	break;
-			case eCursor_arrow_east:			cursor = 140; 	break;
-			case eCursor_arrow_east_plus:	cursor = 141; 	break;
-			case eCursor_crosshair:				cursor = crossCursor; 	break;
-			case eCursor_move:						cursor = 142; 	break;
-			case eCursor_help:						cursor = 143; 	break;
-			case eCursor_copy:						cursor = 144; 	break; // CSS3
-			case eCursor_alias:						cursor = 145; 	break;
-			case eCursor_context_menu:		cursor = 146; 	break;
-			case eCursor_cell:						cursor = plusCursor; 	 break;
-			case eCursor_grab:						cursor = 147; 	break;
-			case eCursor_grabbing:				cursor = 148; 	break;
-			case eCursor_spinning:				cursor = 149; 	break;
-			case eCursor_count_up:				cursor = watchCursor; 	break;
-			case eCursor_count_down:			cursor = watchCursor; 	break;
-			case eCursor_count_up_down:		cursor = watchCursor; 	break;
-	  }
-	  if (cursor > 0)
-	  {
-      if (cursor >= 128) {
-        nsMacResources::OpenLocalResourceFile();
-        CursHandle cursHandle = ::GetCursor(cursor);
-        NS_ASSERTION ( cursHandle, "Can't load cursor, is the resource file installed correctly?" );
-        if ( cursHandle )
-          ::SetCursor(*cursHandle);
-        nsMacResources::CloseLocalResourceFile();
-      }
-      else {
-        CursHandle cursHandle = ::GetCursor(cursor);
-        NS_ASSERTION ( cursHandle, "Can't load cursor, is the resource file installed correctly?" );
-        if ( cursHandle )
-          ::SetCursor(*cursHandle);
-      }
-	  }
+    return NS_OK;
   }
- 
+
+  if ( gCursorSpinner == nsnull && OnJaguarOrLater())
+  {
+      gCursorSpinner = new CursorSpinner();
+  }
+  
+  // mac specific cursor manipulation
+  short cursor = -1;
+  switch (aCursor)
+  {
+    case eCursor_standard:            cursor = kThemeArrowCursor; break;
+    case eCursor_wait:                cursor = kThemeWatchCursor; break;
+    case eCursor_select:              cursor = kThemeIBeamCursor; break;
+    case eCursor_hyperlink:           cursor = kThemePointingHandCursor; break;
+    case eCursor_sizeWE:              cursor = kThemeResizeLeftRightCursor; break;
+    case eCursor_sizeNS:              cursor = 129; break;
+    case eCursor_sizeNW:              cursor = 130; break;
+    case eCursor_sizeSE:              cursor = 131; break;
+    case eCursor_sizeNE:              cursor = 132; break;
+    case eCursor_sizeSW:              cursor = 133; break;
+    case eCursor_arrow_north:         cursor = 134; break;
+    case eCursor_arrow_north_plus:    cursor = 135; break;
+    case eCursor_arrow_south:         cursor = 136; break;
+    case eCursor_arrow_south_plus:    cursor = 137; break;
+    case eCursor_arrow_west:          cursor = 138; break;
+    case eCursor_arrow_west_plus:     cursor = 139; break;
+    case eCursor_arrow_east:          cursor = 140; break;
+    case eCursor_arrow_east_plus:     cursor = 141; break;
+    case eCursor_crosshair:           cursor = kThemeCrossCursor; break;
+    case eCursor_move:                cursor = kThemeOpenHandCursor; break;
+    case eCursor_help:                cursor = 143; break;
+    case eCursor_copy:                cursor = 144; break; // CSS3
+    case eCursor_alias:               cursor = 145; break;
+    case eCursor_context_menu:        cursor = 146; break;
+    case eCursor_cell:                cursor = kThemePlusCursor; break;
+    case eCursor_grab:                cursor = kThemeOpenHandCursor; break;
+    case eCursor_grabbing:            cursor = kThemeClosedHandCursor; break;
+    case eCursor_spinning:            cursor = kSpinCursorFirstFrame; break; // better than kThemeSpinningCursor
+    case eCursor_count_up:            cursor = kThemeCountingUpHandCursor; break;
+    case eCursor_count_down:          cursor = kThemeCountingDownHandCursor; break;
+    case eCursor_count_up_down:       cursor = kThemeCountingUpAndDownHandCursor; break;
+    case eCursor_zoom_in:             cursor = 149; break;
+    case eCursor_zoom_out:            cursor = 150; break;
+    default:                          cursor = kThemeArrowCursor; break;
+  }
+
+  //animated cursors cause crash on Mac OS X 10.1 when Japanese Kotorei input method is enabled
+  if ( OnJaguarOrLater() )
+  {
+    if (aCursor == eCursor_spinning)
+    {
+      gCursorSpinner->StartSpinCursor();
+    }
+    else
+    {
+      gCursorSpinner->StopSpinCursor();
+      nsWindow::SetCursorResource(cursor);
+    }
+  } else {
+    nsWindow::SetCursorResource(cursor);
+  }
+
   return NS_OK;
   
 } // nsWindow::SetCursor
 
+void nsWindow::SetCursorResource(short aCursorResourceNum)
+{
+    if (aCursorResourceNum >= 0)
+    {
+        if (aCursorResourceNum >= 128)
+        {
+            nsMacResources::OpenLocalResourceFile();
+            CursHandle cursHandle = ::GetCursor(aCursorResourceNum);
+            NS_ASSERTION (cursHandle, "Can't load cursor, is the resource file installed correctly?");
+            if (cursHandle)
+            {
+                ::SetCursor(*cursHandle);
+            }
+            nsMacResources::CloseLocalResourceFile();            
+        }
+        else
+        {
+            ::SetThemeCursor(aCursorResourceNum);
+        }
+    }    
+}
+
+CursorSpinner::CursorSpinner() :
+    mSpinCursorFrame(0), mTimerUPP(nsnull), mTimerRef(nsnull)
+{
+   mTimerUPP = NewEventLoopTimerUPP(SpinCursor);
+}
+
+CursorSpinner::~CursorSpinner()
+{
+    if (mTimerRef) ::RemoveEventLoopTimer(mTimerRef);
+    if (mTimerUPP) ::DisposeEventLoopTimerUPP(mTimerUPP);
+}
+
+short CursorSpinner::GetNextCursorFrame()
+{
+    int result = kSpinCursorFirstFrame + mSpinCursorFrame;
+    mSpinCursorFrame = (mSpinCursorFrame + 1) % 4;
+    return (short) result;
+}
+
+void CursorSpinner::StartSpinCursor()
+{
+    OSStatus result = noErr;
+    if (mTimerRef == nsnull)
+    {
+        result = ::InstallEventLoopTimer(::GetMainEventLoop(), 0, 0.25 * kEventDurationSecond,
+                                         mTimerUPP, this, &mTimerRef);
+        if (result != noErr)
+        {
+            mTimerRef = nsnull;
+            nsWindow::SetCursorResource(kSpinCursorFirstFrame);
+        }
+    }
+}
+
+void CursorSpinner::StopSpinCursor()
+{
+    if (mTimerRef)
+    {
+        ::RemoveEventLoopTimer(mTimerRef);
+        mTimerRef = nsnull;
+    }
+}
+
+pascal void CursorSpinner::SpinCursor(EventLoopTimerRef inTimer, void *inUserData)
+{
+    CursorSpinner* cs = reinterpret_cast<CursorSpinner*>(inUserData);
+    nsWindow::SetCursorResource(cs->GetNextCursorFrame());
+}
+
 #pragma mark -
 //-------------------------------------------------------------------------
 //
@@ -1420,7 +1462,7 @@ else
 	StRegionFromPool damagedRgn;
 	if (!damagedRgn)
 		return NS_ERROR_OUT_OF_MEMORY;
-	::GetPortVisibleRegion(GetWindowPort(mWindowPtr), damagedRgn);
+	::GetPortVisibleRegion(::GetWindowPort(mWindowPtr), damagedRgn);
 
 /*
 #ifdef PAINT_DEBUGGING	
@@ -1450,6 +1492,14 @@ else
 
 	if (!::EmptyRgn(updateRgn))
 	{
+
+#if TARGET_CARBON
+    // Ref: http://developer.apple.com/technotes/tn/tn2051.html
+    //      short-circuit QD's implicit LockPortBits()
+    //      Note: MUST unlock before exiting this scope (see below)
+    ::LockPortBits(::GetWindowPort(mWindowPtr));
+#endif
+
 #if DEBUG
 		// measure the time it takes to refresh the window, if the control key is down.
     unsigned long long start, finish;
@@ -1542,6 +1592,14 @@ else
     // Copy updateRgn to regionToValidate
     if (regionToValidate)
       ::CopyRgn(updateRgn, regionToValidate);
+
+#if TARGET_CARBON
+    // Ref: http://developer.apple.com/technotes/tn/tn2051.html
+    //      short-circuit QD's implicit LockPortBits()
+    //      Note: unlock before exiting (locked above)
+    ::UnlockPortBits(::GetWindowPort(mWindowPtr));
+#endif
+
 	}
 
 #if PINK_PROFILING
@@ -1659,13 +1717,9 @@ void nsWindow::UpdateWidget(nsRect& aRect, nsIRenderingContext* aContext)
 		return;
 
 	// initialize the paint event
-	nsPaintEvent paintEvent;
-	paintEvent.eventStructType			= NS_PAINT_EVENT;		// nsEvent
-	paintEvent.message					= NS_PAINT;
-	paintEvent.widget					= this;					// nsGUIEvent
-	paintEvent.nativeMsg				= NULL;
-	paintEvent.renderingContext			= aContext;				// nsPaintEvent
-	paintEvent.rect						= &aRect;
+	nsPaintEvent paintEvent(NS_PAINT, this);
+	paintEvent.renderingContext = aContext;         // nsPaintEvent
+	paintEvent.rect             = &aRect;
 
 	// draw the widget
 	StartDraw(aContext);
@@ -1860,33 +1914,33 @@ nsWindow::ScrollBits ( Rect & inRectToScroll, PRInt32 inLeftDelta, PRInt32 inTop
 //-------------------------------------------------------------------------
 NS_IMETHODIMP nsWindow::Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *aClipRect)
 {
-	if (! mVisible || !ContainerHierarchyIsVisible())
-		return NS_OK;
 
-  nsRect scrollRect;	
+  
+  if (mVisible && ContainerHierarchyIsVisible())  {
+    nsRect scrollRect;	
 
-	// If the clipping region is non-rectangular, just force a full update, sorry.
-  // XXX ?
-	if (!IsRegionRectangular(mWindowRegion)) {
-		Invalidate(PR_TRUE);
-		goto scrollChildren;
-	}
+	  // If the clipping region is non-rectangular, just force a full update, sorry.
+    // XXX ?
+	  if (!IsRegionRectangular(mWindowRegion)) {
+		  Invalidate(PR_TRUE);
+		  goto scrollChildren;
+	  }
 
-	//--------
-	// Scroll this widget
-	if (aClipRect)
-		scrollRect = *aClipRect;
-	else
-	{
-		scrollRect = mBounds;
-		scrollRect.x = scrollRect.y = 0;
-	}
+	  //--------
+	  // Scroll this widget
+	  if (aClipRect)
+		  scrollRect = *aClipRect;
+	  else
+	  {
+		  scrollRect = mBounds;
+		  scrollRect.x = scrollRect.y = 0;
+	  }
 
-	Rect macRect;
-	nsRectToMacRect(scrollRect, macRect);
+	  Rect macRect;
+	  nsRectToMacRect(scrollRect, macRect);
 
 
-	StartDraw();
+	  StartDraw();
 
 		// Clip to the windowRegion instead of the visRegion (note: the visRegion
 		// is equal to the windowRegion minus the children). The result is that
@@ -1897,6 +1951,7 @@ NS_IMETHODIMP nsWindow::Scroll(PRInt32 aDx, PRInt32 aDy, nsRect *aClipRect)
 		ScrollBits(macRect,aDx,aDy);
 
 	EndDraw();
+  }
 
 scrollChildren:
 	//--------
@@ -2057,17 +2112,10 @@ PRBool nsWindow::DispatchMouseEvent(nsMouseEvent &aEvent)
 PRBool nsWindow::ReportDestroyEvent()
 {
 	// nsEvent
-	nsGUIEvent moveEvent;
-	moveEvent.eventStructType = NS_GUI_EVENT;
+	nsGUIEvent moveEvent(NS_DESTROY, this);
 	moveEvent.message			= NS_DESTROY;
-	moveEvent.point.x			= 0;
-	moveEvent.point.y			= 0;
 	moveEvent.time				= PR_IntervalNow();
 
-	// nsGUIEvent
-	moveEvent.widget			= this;
-	moveEvent.nativeMsg		= nsnull;
-
 	// dispatch event
 	return (DispatchWindowEvent(moveEvent));
 }
@@ -2079,17 +2127,11 @@ PRBool nsWindow::ReportDestroyEvent()
 PRBool nsWindow::ReportMoveEvent()
 {
 	// nsEvent
-	nsGUIEvent moveEvent;
-	moveEvent.eventStructType = NS_GUI_EVENT;
-	moveEvent.message			= NS_MOVE;
+	nsGUIEvent moveEvent(NS_MOVE, this);
 	moveEvent.point.x			= mBounds.x;
 	moveEvent.point.y			= mBounds.y;
 	moveEvent.time				= PR_IntervalNow();
 
-	// nsGUIEvent
-	moveEvent.widget			= this;
-	moveEvent.nativeMsg		= nsnull;
-
 	// dispatch event
 	return (DispatchWindowEvent(moveEvent));
 }
@@ -2101,17 +2143,9 @@ PRBool nsWindow::ReportMoveEvent()
 PRBool nsWindow::ReportSizeEvent()
 {
 	// nsEvent
-	nsSizeEvent sizeEvent;
-	sizeEvent.eventStructType = NS_SIZE_EVENT;
-	sizeEvent.message			= NS_SIZE;
-	sizeEvent.point.x			= 0;
-	sizeEvent.point.y			= 0;
+	nsSizeEvent sizeEvent(NS_SIZE, this);
 	sizeEvent.time				= PR_IntervalNow();
 
-	// nsGUIEvent
-	sizeEvent.widget			= this;
-	sizeEvent.nativeMsg		= nsnull;
-
 	// nsSizeEvent
 	sizeEvent.windowSize	= &mBounds;
 	sizeEvent.mWinWidth		= mBounds.width;
@@ -2150,12 +2184,19 @@ void nsWindow::CalcWindowRegions()
 	{
 		while (parent && (!parent->mIsTopWidgetWindow))
 	{
-		if (parent->mWindowRegion)
-		{
-			::OffsetRgn(parent->mWindowRegion, origin.x, origin.y);
-			::SectRgn(mWindowRegion, parent->mWindowRegion, mWindowRegion);
-			::OffsetRgn(parent->mWindowRegion, -origin.x, -origin.y);
-		}
+    if (parent->mWindowRegion)
+    {
+      // Under 10.2, if we offset a region beyond the coordinate space,
+      // OffsetRgn() will silently fail and restoring it will then cause the
+      // widget to be out of place (visible as 'shearing' when scrolling).
+      // To prevent that, we copy the original region and work on that. (bug 162885)
+      StRegionFromPool shiftedParentWindowRgn;
+      if ( !shiftedParentWindowRgn )
+        return;
+      ::CopyRgn(parent->mWindowRegion, shiftedParentWindowRgn); 
+      ::OffsetRgn(shiftedParentWindowRgn, origin.x, origin.y);
+      ::SectRgn(mWindowRegion, shiftedParentWindowRgn, mWindowRegion);
+    }
 		origin.x -= parent->mBounds.x;
 		origin.y -= parent->mBounds.y;
 		parent = (nsWindow*)parent->mParent;
@@ -2384,7 +2425,7 @@ NS_IMETHODIMP nsWindow::WidgetToScreen(const nsRect& aLocalRect, nsRect& aGlobal
 NS_IMETHODIMP nsWindow::ScreenToWidget(const nsRect& aGlobalRect, nsRect& aLocalRect)
 {
 	aLocalRect = aGlobalRect;
-	nsIWidget* theParent = this->GetParent();
+	nsIWidget* theParent = GetParent();
 	if ( theParent ) {
 		// Recursive case
 		//
@@ -2422,7 +2463,7 @@ NS_IMETHODIMP nsWindow::ScreenToWidget(const nsRect& aGlobalRect, nsRect& aLocal
  *  @param   aRect -- The nsRect that is the source
  *  @param   aMacRect -- The Mac Rect destination
  */
-void nsWindow::nsRectToMacRect(const nsRect& aRect, Rect& aMacRect) const
+void nsWindow::nsRectToMacRect(const nsRect& aRect, Rect& aMacRect)
 {
 		aMacRect.left = aRect.x;
 		aMacRect.top = aRect.y;
@@ -2474,9 +2515,9 @@ NS_IMETHODIMP nsWindow::SetTitle(const nsString& title)
   return NS_OK;
 }
 
-NS_IMETHODIMP nsWindow::GetAttention()
+NS_IMETHODIMP nsWindow::GetAttention(PRInt32 aCycleCount)
 {
-	// Since the Mac doesn't consider each window a seperate process this call functions
+        // Since the Mac doesn't consider each window a separate process this call functions
 	// slightly different than on other platforms.  We first check to see if we're the
 	// foreground process and, if so, ignore the call.  We also check to see if a notification
 	// is already pending and, if so, remove it since we only have one notification per process.
@@ -2514,6 +2555,99 @@ NS_IMETHODIMP nsWindow::GetAttention()
 #pragma mark -
 
 
+NS_IMETHODIMP nsWindow::GetPluginClipRect(nsRect& outClipRect, nsPoint& outOrigin, PRBool& outWidgetVisible)
+{
+  PRBool isVisible = mVisible;
+
+  nsRect widgetClipRect = mBounds;
+  // absX and absY are top-level window-relative coordinates
+  nscoord absX = widgetClipRect.x;
+  nscoord absY = widgetClipRect.y;
+
+  nscoord ancestorX = -widgetClipRect.x;
+  nscoord ancestorY = -widgetClipRect.y;
+
+  // Calculate clipping relative to this widget
+  widgetClipRect.x = 0;
+  widgetClipRect.y = 0;
+
+  // Gather up the absolute position of the widget, clip window, and visibilty
+  nsCOMPtr<nsIWidget> widget = getter_AddRefs(GetParent());
+  while (widget)
+  {
+    if (isVisible)
+      widget->IsVisible(isVisible);
+
+    nsRect widgetRect;
+    widget->GetClientBounds(widgetRect);
+    nscoord wx = widgetRect.x;
+    nscoord wy = widgetRect.y;
+
+    widgetRect.x = ancestorX;
+    widgetRect.y = ancestorY;
+
+    widgetClipRect.IntersectRect(widgetClipRect, widgetRect);
+    absX += wx;
+    absY += wy;
+    widget = getter_AddRefs(widget->GetParent());
+    if (!widget)
+    {
+      // Don't include the top-level windows offset
+      // printf("Top level window offset %d %d\n", wx, wy);
+      absX -= wx;
+      absY -= wy;
+    }
+    ancestorX -= wx;
+    ancestorY -= wy;
+  }
+
+  widgetClipRect.x += absX;
+  widgetClipRect.y += absY;
+
+  outClipRect = widgetClipRect;
+  outOrigin.x = absX;
+  outOrigin.y = absY;
+  
+  outWidgetVisible = isVisible;
+  return NS_OK;
+}
+
+
+NS_IMETHODIMP nsWindow::StartDrawPlugin(void)
+{
+  // setup the port background color for the plugin
+  GrafPtr savePort;
+  ::GetPort(&savePort);  // save our current port
+  ::SetPortWindowPort(mWindowPtr);
+  
+  RGBColor macColor;
+  macColor.red   = COLOR8TOCOLOR16(NS_GET_R(mBackground));  // convert to Mac color
+  macColor.green = COLOR8TOCOLOR16(NS_GET_G(mBackground));
+  macColor.blue  = COLOR8TOCOLOR16(NS_GET_B(mBackground));
+  ::RGBBackColor(&macColor);
+
+  ::SetPort(savePort);  // restore port
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP nsWindow::EndDrawPlugin(void)
+{
+  GrafPtr savePort;
+  ::GetPort(&savePort);  // save our current port
+  ::SetPortWindowPort(mWindowPtr);
+
+  RGBColor rgbWhite = { 0xFFFF, 0xFFFF, 0xFFFF };
+  ::RGBBackColor(&rgbWhite);
+
+  ::SetPort(savePort);  // restore port
+  return NS_OK;
+}
+
+
+#pragma mark -
+
+
 NS_IMETHODIMP nsWindow::ResetInputState()
 {
 	// currently, the nsMacEventHandler is owned by nsMacWindow, which is the top level window
