@@ -268,7 +268,7 @@ DeletionTest(const char* creationPath, const char* appendPath, PRBool recursive)
 {
     nsCOMPtr<nsILocalFile> file;
     nsresult rv = 
-      nsComponentManager::CreateInstance(NS_FILE_CONTRACTID, NULL,
+      nsComponentManager::CreateInstance(NS_LOCAL_FILE_CONTRACTID, NULL,
 					 NS_GET_IID(nsILocalFile),
 					 (void**)getter_AddRefs(file));
     
@@ -312,6 +312,45 @@ DeletionTest(const char* creationPath, const char* appendPath, PRBool recursive)
     
 }
 
+void 
+MoveTest(const char *testFile, const char *targetDir)
+{
+  Banner("Move Test");
+
+  printf("start move test\n");
+
+  nsresult rv;
+  nsCOMPtr<nsILocalFile> file(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
+     
+  if (!file) 
+  {
+    printf("create nsILocalFile failed\n");
+    return;
+  }
+
+  rv = file->InitWithNativePath(nsDependentCString(testFile));
+  VerifyResult(rv);
+  
+  nsCOMPtr<nsILocalFile> dir(do_CreateInstance(NS_LOCAL_FILE_CONTRACTID));
+ 
+  if (!dir) 
+  {
+    printf("create nsILocalFile failed\n");
+    return;
+  }
+
+  rv = dir->InitWithNativePath(nsDependentCString(targetDir));
+  VerifyResult(rv);
+
+  rv = file->MoveToNative(dir, NS_LITERAL_CSTRING("newtemp"));
+  VerifyResult(rv);
+  if (NS_FAILED(rv))
+  {
+    printf("MoveToNative() test Failed.\n");
+  }
+  printf("end move test\n");
+}
+
 
 
 int main(void)
@@ -322,13 +361,15 @@ int main(void)
     NS_ASSERTION(registrar, "Null nsIComponentRegistrar");
     registrar->AutoRegister(nsnull);
   
-#ifdef XP_PC
+#if defined(XP_WIN) || defined(XP_OS2)
     InitTest("c:\\temp\\", "sub1/sub2/");
     InitTest("d:\\temp\\", "sub1\\sub2\\");
 
     CreationTest("c:\\temp\\", "file.txt", nsIFile::NORMAL_FILE_TYPE, 0644);
     DeletionTest("c:\\temp\\", "file.txt", PR_FALSE);
 
+    MoveTest("c:\\newtemp\\", "d:");
+
     CreationTest("c:\\temp\\", "mumble\\a\\b\\c\\d\\e\\f\\g\\h\\i\\j\\k\\", nsIFile::DIRECTORY_TYPE, 0644);
     DeletionTest("c:\\temp\\", "mumble", PR_TRUE);
 
@@ -353,6 +394,6 @@ int main(void)
     CopyTest("/tmp/test.txt", "/tmp/foo");
 
 #endif /* XP_UNIX */
-#endif /* XP_PC */
+#endif /* XP_WIN || XP_OS2 */
     return 0;
 }
